################################################################################
################################################################################
####                                                                        ####
#### File 1 of replication code for:                                        ####
####   Park, Sunhee and David J. Hendry. "Reassessing Schoenfeld Residual   ####
####     Tests of Proportional Hazards in Political Science Event History   ####
####     Analyses." American Journal of Political Science.                  ####
####                                                                        ####
####      - This file creates Figure 1 from the main text and Figure SI.C.1 ####
####        from the online Supporting Information and saves them in the    ####
####        directory "figures".                                            ####
####        *** Note: The actual replication datasets are not provided      ####
####            because they are datasets from previously published work    ####
####            and the original authors and journals maintain sole rights  ####
####            to their distribution.  In all cases, the datasets referred ####
####            to herein were made publicly available by the original      ####
####            authors or were transmitted through personal                ####
####            correspondence.  Please contact Park and Hendry or the      ####
####            original authors for information about how to obtain them.  ####
####                                                                        ####
################################################################################
################################################################################

#set.wd(...)

library(survival)
library(foreign)
library(car)
# For Latex-style font in R graphical devices
#library(extrafont)
#loadfonts()
#Sys.setenv(R_GSCMD = 'INSERT GHOSTSCRIPT PATH NAME HERE')
#font_install('fontcm')

################################################################################
## Figure 1 - Cunningham 2011 and Maeda 2010 outlier examples
################################################################################

#Cunningham 2011 - Model 4
cunningham2011 <-
  read.dta('data/nonpublic/cunningham2011/APSR duration analysis final.dta')
surv <- with(cunningham2011, Surv(duration, concessionend_l))
cunningham2011.cox <- coxph(surv ~ logfactions + newcombosize + base + 
  logpop + loggdppc + milexpc + strata(numberconcession) + cluster(caseid), 
  method = 'breslow', data = cunningham2011)
cunningham2011.phtest <- cox.zph(cunningham2011.cox, transform = 'identity')
cunningham2011.cox.phlm <-
  lm(cunningham2011.phtest$x ~ cunningham2011.phtest$y[, 3])

#Maeda 2010 - Model 1
maeda2010 <- read.dta('data/nonpublic/maeda2010/Maeda2010JOP_TwoModes.dta')
maeda2010$t0 <- NA
for ( i in unique(maeda2010$demid) ) {
  ifelse(length(maeda2010[maeda2010$demid == i, ]$demmonth) > 1,
    maeda2010[maeda2010$demid == i, ]$t0 <-
      with(maeda2010, c(0, maeda2010[demid == i, ]$demmonth[
      1:(length(maeda2010[demid == i, ]$demmonth) - 1)])),
    maeda2010[maeda2010$demid == i, ]$t0 <- 0)
}
maeda2010$demend2 <- maeda2010$demend
maeda2010[maeda2010$demend == 9, ]$demend2 <- 0
maeda2010$endmode2 <- maeda2010$endmode
maeda2010[maeda2010$endmode == 2, ]$endmode2 <- 0
maeda2010$endmode3 <- 0
maeda2010[maeda2010$endmode == 2, ]$endmode3 <- 1
surv <- with(maeda2010, Surv(t0, demmonth, demend2))
maeda2010.cox <- coxph(surv ~ dev + gro + presi + mixed + majgovm + ethnic + 
  ope + urb + postcw + impose + prereg_ind + prereg_mil + regiondem +
  cluster(country), data = maeda2010)
maeda2010.phtest <- cox.zph(maeda2010.cox, transform = 'identity')
maeda2010.cox.phlm <- lm(maeda2010.phtest$x ~ maeda2010.phtest$y[, 11])

## Output Figure 1 as pdf
#pdf(file = 'figures/figure-01.pdf', family = 'CM Roman', height = 3.5)
pdf(file = 'figures/figure-01.pdf', height = 3.5)
  par(mfrow = c(1, 2), mar = c(5, 4, 1, 1))
  plot(cunningham2011.phtest[3], ann = FALSE)
  title(xlab = bquote(italic(t)), 
    ylab = bquote(beta(italic(t)) * ' for Territorial Base'), line = 2.5)
  text(13, 3.16, 'From Cunningham (2011),\nTable 5, Model 4')
  plot(maeda2010.phtest[11], ann = FALSE)
  title(xlab = bquote(italic(t)),
    ylab = bquote(beta(italic(t)) * ' for Colony*'), line = 2.5)
  text(290, 24, 'From Maeda (2010)\nTable 1, Model 1')
dev.off()
#embed_fonts('figures/figure-01.pdf')

cunningham.xaxis <- as.numeric(row.names(cunningham2011.cox.phlm$model))
cunningham.stud <- rstudent(cunningham2011.cox.phlm)
cunningham.bonf <- pmin(outlierTest(cunningham2011.cox.phlm, order = FALSE,
  n.max = length(cunningham.xaxis), cutoff = length(cunningham.xaxis))$bonf.p, 
  1)

maeda.xaxis <- as.numeric(row.names(maeda2010.cox.phlm$model))
maeda.stud <- rstudent(maeda2010.cox.phlm)
maeda.bonf <- pmin(outlierTest(maeda2010.cox.phlm, order = FALSE,
  n.max = length(maeda.xaxis), cutoff = length(maeda.xaxis))$bonf.p, 1)


################################################################################
## Figure SI.C1 - Cunningham 2011 and Maeda 2010 studentized residual tests
################################################################################

## Output Figure SI.C1 as pdf
#pdf(file = 'figures/figure-si-c01.pdf', family = 'CM Roman', height = 3.5)
pdf(file = 'figures/figure-si-c01.pdf', height = 3.5)
  par(mfrow = c(2, 2))
  par(mar = c(2, 4.5, 2, 1))
  plot(cunningham.xaxis, cunningham.stud, type = 'n', ylab = '', xlab = '',
    xaxt = 'n', yaxt = 'n', main = '')
  mtext('Cunningham (2011), Table 5, Model 4', 3, line = .5)
  grid(NULL, NULL, lty = 1, col = 'cornsilk2')
  abline(h = 0, lty = 2)
  lines(cunningham.xaxis, cunningham.stud, type = 'h')
  points(cunningham.xaxis, cunningham.stud)
  box()
  axis(1, labels = FALSE)
  axis(2, labels = TRUE, las = 1)
  text(29, 3, 'Survival Times for\nTerritorial Base')
  mtext('Studentized', side = 2, line = 3.25)
  mtext('Residuals', side = 2, line = 2.25)

  plot(maeda.xaxis, maeda.stud, type = 'n', ylab = '', xlab = '', xaxt = 'n',
    yaxt = 'n', main = '')
  mtext('Maeda (2010), Table 1, Model 1', 3, line = .5)
  grid(NULL, NULL, lty = 1, col = 'cornsilk2')
  abline(h = 0, lty = 2)
  lines(maeda.xaxis, maeda.stud, type = 'h')
  points(maeda.xaxis, maeda.stud)
  box()
  axis(1, labels = FALSE)
  axis(2, labels = TRUE, las = 1)
  text(15, 2.5, 'Survival Times\nfor Colony')
  mtext('Studentized', side = 2, line = 3.25)
  mtext('Residuals', side = 2, line = 2.25)

  par(mar = c(4, 4.5, 0, 1))
  plot(cunningham.xaxis, cunningham.bonf, type = 'n', ylab = '', xlab = '',
    xaxt = 'n', yaxt = 'n', ylim = c(0, 1))
  grid(NULL, NULL, lty = 1, col = 'cornsilk2')
  for ( i in 1:length(cunningham.xaxis) ) {
    segments(cunningham.xaxis[i], 1, cunningham.xaxis[i], cunningham.bonf[i])
  }
  points(cunningham.xaxis, cunningham.bonf)
  box()
  axis(1, labels = TRUE)
  axis(2, labels = TRUE, las = 1)
  mtext('Bonferroni', side = 2, line = 3.25)
  mtext(expression(italic(p) * '-value'), side = 2, line = 2.25)
  mtext('Case Number', side = 1, line = 2.5)

  plot(maeda.xaxis, maeda.bonf, type = 'n', ylab = '', xlab = '', xaxt = 'n', 
    yaxt = 'n', ylim = c(0, 1))
  grid(NULL, NULL, lty = 1, col = 'cornsilk2')
  for ( i in 1:length(maeda.xaxis) ) {
    segments(maeda.xaxis[i], 1, maeda.xaxis[i], maeda.bonf[i])
  }
  points(maeda.xaxis, maeda.bonf)
  box()
  axis(1, labels = TRUE)
  axis(2, labels = TRUE, las = 1)
  mtext('Bonferroni', side = 2, line = 3.25)
  mtext(expression(italic(p) * '-value'), side = 2, line = 2.25)
  mtext('Case Number', side = 1, line = 2.5)
dev.off()
#embed_fonts('figures/figure-si-c01.pdf')

# Clearing workspace
rm(list = ls())
