################################################################################
################################################################################
####                                                                        ####
#### File 2 of replication code for:                                        ####
####   Park, Sunhee and David J. Hendry. "Reassessing Schoenfeld Residual   ####
####     Tests of Proportional Hazards in Political Science Event History   ####
####     Analyses." American Journal of Political Science.                  ####
####                                                                        ####
####      - This file performs the simulations reported in the main text    ####
####        and saves datasets and related materials for each simulation    ####
####        scenario as an individual R workspace in the directory          ####
####        "r_workspaces".                                                 ####
####        *** Note: These simulations will likely take several hours to   ####
####            run on most Windows, Mac, and Linux personal computing      ####
####            systems at the time of publication.                         ####
####                                                                        ####
################################################################################
################################################################################

#set.wd(...)

library(survival)
library(msm)

################################################################################
## Simulations
################################################################################

## Creating variables, functions, and parameters to be used in all simulations

# Creating g() AND g^-1()
g <- function(x) {
  x^2
}
g.inv <- sqrt

# Creating the time scale and transformed time scale
t <- 0:299
g.inv.t <- g.inv(t)

# Creating the bounds of truncation
t.max <- 150
t.min <- 10
g.inv.t.max <- g.inv(t.max)
g.inv.t.min <- g.inv(t.min)

# Data generating process for the first covariate
B <- function(N, m, M) {
  runif(N, m, M)
}
# Data generating process for the second covariate
H <- function(N, trials, p) {
  rbinom(N, trials, p)
}

# Number of observations
n <- 500
# Number of simulated datasets
sims <- 1000
# Number of outliers
num.outliers <- 5

# Creating function necessary to generate truncated piecewise exponential random
#   variables
k <- function(x, m, M, rates, t) {
  # Calculates the density of a piecewise exponential distribution with
  #   user-defined rates and upper and lower bounds.  If the value of the
  #   input falls between the upper and lower bound, inclusive, function
  #   returns density.  If the value of the input is lower than the lower bound
  #   or higher than the upper bound, the function returns 0.  Employs function
  #   'dpexp' from 'msm' package.
  #
  # Args:
  #   x:     random draw from a piecewise exponential distribution with rates
  #          defined by 'rates' and time scale defined by 't'
  #   m:     lower bound of truncation
  #   M:     upper bound of truncation
  #   rates: rates of piecewise exponential distribution
  #   t:     time scale
  ifelse (x <= m | x >= M, 0, dpexp(x, rates, t))
}

#BETA
b1 <- rep(NA, length(t))
b2 <- rep(NA, length(t))
for ( i in 1:length(t) ) {
  ifelse(t[i] < 10, b1[i] <- 0.1,
    ifelse(t[i] < 15, b1[i] <- 1,
      ifelse(t[i] < 20, b1[i] <- 2, b1[i] <- 3)))
  ifelse(t[i] < 10, b2[i] <- -5,
    ifelse(t[i] < 15, b2[i] <- -3, b2[i] <- -1))
}

# Creating function necessary to generate truncated piecewise exponential random
#   variables
gen.y <- function(x, g.inv.t, g.inv.t.min, g.inv.t.max) {
  # Generates truncated piecewise exponential random variables with user-defined
  #   rates and upper and lower bounds.  Employs functions 'dpexp', 'ppexp',
  #   and 'rpexp' from 'msm' package and 'k' function defined above.
  #
  # Args:
  #   x:           matrix whose final column is a set of rates for a piecewise
  #                exponential random variable
  #   g.inv.t:     inverse of chosen g function evaluated over t
  #   g.inv.t.min: chosen lower bound for survival times
  #   g.inv.t.max: chosen upper bound for survival times
  rates <- x[, ncol(x)]
  d <- ppexp(g.inv.t.max, rates, g.inv.t) - ppexp(g.inv.t.min, rates, g.inv.t)
  M <- 1 / d
  r <- 20
  repeat {
    y <- rpexp(r, rates, g.inv.t)
    u <- runif(r)
    t <- M * ((k(y, g.inv.t.min, g.inv.t.max, rates, g.inv.t) / d / dpexp(y,
         rates, g.inv.t)))
    y <- y[u <= t][1]
    if (!is.na(y)) break
  }
  y
}

################################################################################
## Simulation 1: 0 percent censoring
################################################################################

# Defining proportion of censored cases
prop.cen <- 0

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(154118926)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i] - 1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation01.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y)


################################################################################
## Simulation 2: 50 percent censoring, random censoring
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.5

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(552413383)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i] - 1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation02.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y)


################################################################################
## Simulation 3: 25 percent censoring, random censoring
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.25

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(416292795)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation03.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y)


################################################################################
## Simulation 4: 10 percent censoring, random censoring
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.1

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(966153648)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation04.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y)


################################################################################
## Simulation 5: 50 percent censoring, long censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.5

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(549665666)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation05.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)


################################################################################
## Simulation 6: 25 percent censoring, long censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.25

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(435887247)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation06.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)


################################################################################
## Simulation 7: 10 percent censoring, lng censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.1

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(737624579)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation07.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)


################################################################################
## Simulation 8: 50 percent censoring, short censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.5

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(846397972)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation08.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)


################################################################################
## Simulation 9: 25 percent censoring, short censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.25

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(868623183)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation09.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)


################################################################################
## Simulation 10: 10 percent censoring, short censored times
################################################################################

# Defining proportion of censored cases
prop.cen <- 0.1

# Defining empty lists for simulated vectors of data, estimated models, and
#   Schoenfeld residual tests of proportional hazards
data.list <- list()
model.list <- list()
sch.id.list <- list()
sch.ln.list <- list()
sch.km.list <- list()
sch.ra.list <- list()

# Recording initial time to measure elapsed time for simulation
start.t <- Sys.time()

# Setting seed value
set.seed(815963994)

# Main loop for data generation
for ( j in 1:sims ) {

  # Defining empty lists for simulated covariate vector and piecewise
  #   exponential rate parameters
  z.list <- list()
  for ( i in 1:n ) {
    # Generating covariates according to predifined DGPs
    z1 <- B(length(t), -0.5, 0.5)
    z2 <- H(length(t), 1, 0.5)
    # Adding covariate and piecewise exponential rates to list
    z.list[[i]] <- cbind(z1, z2, exp(b1 * z1 + b2 * z2))
  }
  # Generating y as piecewise exponential conditional on z
  y <- sapply(z.list, gen.y, g.inv.t = g.inv.t, g.inv.t.min = g.inv.t.min,
    g.inv.t.max = g.inv.t.max)
  # Calculating g(y)
  g.y <- g(y)
  med.t.scale <- 0:ceiling(median(0:ceiling(max(g.y))))
  delta <- sample(med.t.scale, num.outliers, replace = TRUE)
  for ( i in 1:num.outliers ) {
    g.y[g.y == sort(g.y, TRUE)[i]] <- g.y[g.y == sort(g.y, TRUE)[i]] + delta[i]
  }

  # Creating random censoring indicator
  d <- sample(0:1, n, replace = TRUE, prob = c(prop.cen, 1 - prop.cen))

  d1 <- d[d == 1]
  d0 <- d[d == 0]

  d.biased <- rep(NA, n)
  for ( i in 1:n ) {
    if (g.y[i] <= quantile(g.y)[2]) {
      success <- rbinom(1, 1, 0.15)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] > quantile(g.y)[2] & g.y[i] < quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.5)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
    if (g.y[i] >= quantile(g.y)[4]) {
      success <- rbinom(1, 1, 0.85)
      if (length(d1) > 0 & length(d0) > 0) {
        ifelse (success == 1,
          c(d.temp <- d1[1], d1 <- d1[-1]),
          c(d.temp <- d0[1], d0 <- d0[-1]))
      }
      if (length(d1) > 0 & length(d0) == 0) {
        d.temp <- d1[1]
        d1 <- d1[-1]
      }
      if (length(d1) == 0 & length(d0) > 0) {
        d.temp <- d0[1]
        d0 <- d0[-1]
      }
      d.biased[i] <- d.temp
    }
  }

  # Creating corrected dataset
  data <- NULL
  for ( i in 1:n ) {
    id.temp <- rep(i, ceiling(g.y[i]))
    time.temp <- c(1:ceiling(g.y[i]))
    time0.temp <- 0:ceiling(g.y[i]-1)
    d.temp <- c(rep(0, length(time.temp) - 1), d[i])
    z.temp <- cbind(z.list[[i]][1:(ceiling(g.y[i])), 1],
      z.list[[i]][1:(ceiling(g.y[i])), 2])
    data.temp <- cbind(id.temp, time.temp, time0.temp, d.temp, z.temp)
    data <- rbind(data, data.temp)
  }
  colnames(data) <- c('id', 't', 't0', 'd', 'z1', 'z2')
  data <- data.frame(data)

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data)

  # Saving results
  data.list[[j]] <- data
  model.list[[j]] <- model

}

# Recording elapsed time for simulation procedure
elapsed.time <- Sys.time() - start.t

# Scaled Schoenfeld residual tests
for ( j in 1:sims ) {

  # Estimating basic Cox model using data
  model <- coxph(Surv(t0, t, d) ~ z1 + z2, data = data.list[[j]])

  # Performing Schoenfeld residual tests of proportional hazards assumption
  sch.id <- cox.zph(model, transform = 'identity')
  sch.ln <- cox.zph(model, transform = 'log')
  sch.km <- cox.zph(model, transform = 'km')
  sch.ra <- cox.zph(model, transform = 'rank')

  # Saving results
  sch.id.list[[j]] <- sch.id
  sch.ln.list[[j]] <- sch.ln
  sch.km.list[[j]] <- sch.km
  sch.ra.list[[j]] <- sch.ra

}

# Removing unneeded objects from workspace
rm(d, d.temp, data.temp, data, i, id.temp, j, model, start.t, sch.id, sch.km,
  sch.ln, sch.ra, time.temp, time0.temp, y, z1, z2, z.list, z.temp)

save.image('r_workspaces/simulation10.RData')

# Clearing lists of simulated vectors of data, estimated models, and Schoenfeld
#   residual tests of proportional hazards
rm(data.list, model.list, sch.id.list, sch.ln.list, sch.km.list, sch.ra.list,
  elapsed.time, prop.cen, delta, med.t.scale, g.y, d.biased, d0, d1, success)

# Clearing simulation functions and parameters
#rm(B, b1, b2, g, g.inv, g.inv.t, g.inv.t.max, g.inv.t.min, gen.y, H, k, n,
#  num.outliers, sims, t, t.max, t.min)
# Clearing workspace
rm(list = ls())
