################################################################################
################################################################################
####                                                                        ####
#### File 3 of replication code for:                                        ####
####   Park, Sunhee and David J. Hendry. "Reassessing Schoenfeld Residual   ####
####     Tests of Proportional Hazards in Political Science Event History   ####
####     Analyses." American Journal of Political Science.                  ####
####                                                                        ####
####      - This file produces custom functions to be used in the analysis  ####
####        of both the simulation and replication datasets.                ####
####                                                                        ####
################################################################################
################################################################################

################################################################################
## Custom functions for use in later analyses
################################################################################

zphTable <- function(cox.fit, digits) {
  # Performs cox.zph function from the survival package using the four
  #   available time transformations and outputs one table with side-by-side
  #   presentation of results
  # Args:
  #   cox.fit: fit of a Cox model using the coxph function
  #   digits:  desired number of digits for presentation of results
  tab <- round(cbind(cox.zph(cox.fit, transform = 'identity')$table,
    cox.zph(cox.fit, transform = 'log')$table, 
    cox.zph(cox.fit, transform = 'km')$table,
    cox.zph(cox.fit, transform = 'rank')$table), digits = digits)
  cols <- colnames(cox.zph(cox.fit)$table)
  colnames(tab) <- c(paste('id.', cols, sep = ''), 
    paste('log.', cols, sep = ''), paste('km.', cols, sep = ''),
    paste('rank.', cols, sep = ''))    
  tab
}

zphPlots <- function(cox.fit, var.labs, var.labs.pretty, directory) {
  # Performs plotting function applied to cox.zph objects from the survival
  #   package using the four available time transformations and outputs four-
  #   panel plots for each model covariate using each transformation; places a
  #   '*' after the y-axis label for covariates that violate the proportional
  #   hazards assumption using a p-value threshold of .05.
  # Args:
  #   cox.fit:         fit of a Cox model using the coxph function
  #   var.labs:        variable labels that match covariate names from coxph fit
  #   var.labs.pretty: variable labels to be displayed on y-axis of plot; must
  #                    match length of var.labs
  #   directory:       path for folder in which plots will be placed
  zph.id <- cox.zph(cox.fit, transform = 'identity')
  zph.log <- cox.zph(cox.fit, transform = 'log')
  zph.km <- cox.zph(cox.fit, transform = 'km')
  zph.rank <- cox.zph(cox.fit, transform = 'rank')
  ifelse (length(var.labs) > 1, 
    seq.temp <- 1:(length(var.labs) - 1), seq.temp <- 1)
  for ( i in seq.temp ) {
    #pdf(file = paste(directory, '/', var.labs[i], '.pdf', sep = ''),
    #  family = 'CM Roman')
    pdf(file = paste(directory, '/', var.labs[i], '.pdf', sep = ''))
    par(mfrow = c(2, 2), mar = c(5, 4, 1, 1), cex.axis = 1.2)
    plot(zph.id[i], ann = FALSE)
    ifelse ( zph.id$table[i, 3] < 0.05, sig <- '*', sig <- '' )
    title(xlab = list(bquote(italic(t)), cex = 1.2), line = 2.75)
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.labs.pretty[i]) *
      .(sig)), cex = 1.2), line = 2.25)
    plot(zph.log[i], ann = FALSE)
    ifelse ( zph.log$table[i, 3] < 0.05, sig <- '*', sig <- '' )
    title(xlab = list(bquote('ln(' * italic(t) * ')'), cex = 1.2), line = 2.75)
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.labs.pretty[i]) *
      .(sig)), cex = 1.2), line = 2.25)
    plot(zph.km[i], ann = FALSE)
    ifelse ( zph.km$table[i, 3] < 0.05, sig <- '*', sig <- '' )
    title(xlab = list(bquote('1-KM(' * italic(t) * ')'), cex = 1.2),
      line = 2.75)
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.labs.pretty[i]) *
      .(sig)), cex = 1.2), line = 2.25)
    plot(zph.rank[i], ann = FALSE)
    ifelse ( zph.rank$table[i, 3] < 0.05, sig <- '*', sig <- '' )
    title(xlab = list(bquote('Rank(' * italic(t) * ')'), cex = 1.2), 
      line = 2.75)
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.labs.pretty[i]) *
      .(sig)), cex = 1.2), line = 2.25)
    dev.off()
    #embed_fonts(paste(directory, '/', var.labs[i], '.pdf', sep = ''))
  }
}

zphPlot1 <- function(cox.fit, var.lab, var.lab.pretty, directory, fig.name) {
  # Performs plotting function applied to cox.zph objects from the survival
  #   package using the four available time transformations and outputs a four-
  #   panel plot for a chosen covariate using each one; places a '*' after the
  #   y-axis label for covariates that violate the proportional hazards
  #   assumption using a p-value threshold of .05.
  # Args:
  #   cox.fit:        fit of a Cox model using the coxph function
  #   var.lab:        variable label that matches covariate name for chosen
  #                   covariate from coxph fit
  #   var.lab.pretty: variable label to be displayed on y-axis of plot
  #   directory:      path for folder in which plot will be placed
  #   fig.name:       desired name of figure to be saved
  zph.id <- cox.zph(cox.fit, transform = 'identity')
  zph.log <- cox.zph(cox.fit, transform = 'log')
  zph.km <- cox.zph(cox.fit, transform = 'km')
  zph.rank <- cox.zph(cox.fit, transform = 'rank')
  var.num.seq <- 1:length(rownames(zph.id$table))
  var.num <- var.num.seq[rownames(zph.id$table) == var.lab]
  #pdf(file = paste(directory, '/', fig.name, '.pdf', sep = ''),
  #  family = 'CM Roman')
  pdf(file = paste(directory, '/', fig.name, '.pdf', sep = ''))
  par(mfrow = c(2, 2), mar = c(5, 4, 1, 1), cex.axis = 1.2)
  plot(zph.id[var.num], ann = FALSE)
  ifelse ( zph.id$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote(italic(t)), cex = 1.2), line = 2.75)
  title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.lab.pretty) *
    .(sig)), cex = 1.2), line = 2.25)
  plot(zph.log[var.num], ann = FALSE)
  ifelse ( zph.log$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('ln(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.lab.pretty) *
    .(sig)), cex = 1.2), line = 2.25)
  plot(zph.km[var.num], ann = FALSE)
  ifelse ( zph.km$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('1-KM(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.lab.pretty) *
    .(sig)), cex = 1.2), line = 2.25)
  plot(zph.rank[var.num], ann = FALSE)
  ifelse ( zph.rank$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('Rank(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  title(ylab = list(bquote(beta(italic(t)) * ' for ' * .(var.lab.pretty) *
    .(sig)), cex = 1.2), line = 2.25)
  dev.off()
  #embed_fonts(paste(directory, '/', fig.name, '.pdf', sep = ''))
}

zphPlot2 <- function(cox.fit, z.num, directory, fig.name) {
  # Performs plotting function applied to cox.zph objects from the survival
  #   package using the four available time transformations and outputs a four-
  #   panel plot for a chosen covariate using each one; places a '*' after the
  #   y-axis label for covariates that violate the proportional hazards
  #   assumption using a p-value threshold of .05.  Function is specific to the
  #   labels used for the simulation covariates.
  # Args:
  #   cox.fit:        fit of a Cox model using the coxph function
  #   z.num:          covariate number from two-variable simulations (should be
  #                   1 or 2
  #   directory:      path for folder in which plot will be placed
  #   fig.name:       desired name of figure to be saved
  zph.id <- cox.zph(cox.fit, transform = 'identity')
  zph.log <- cox.zph(cox.fit, transform = 'log')
  zph.km <- cox.zph(cox.fit, transform = 'km')
  zph.rank <- cox.zph(cox.fit, transform = 'rank')
  var.lab = paste('z', z.num, sep = '')
  var.num.seq <- 1:length(rownames(zph.id$table))
  var.num <- var.num.seq[rownames(zph.id$table) == var.lab]
  #pdf(file = paste(directory, '/', var.labs[i], '.pdf', sep = ''),
  #  family = 'CM Roman')
  pdf(file = paste(directory, '/', fig.name, '.pdf', sep = ''))
  par(mfrow = c(2, 2), mar = c(5, 4, 1, 1), cex.axis = 1.2)
  plot(zph.id[var.num], ann = FALSE)
  ifelse ( zph.id$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote(italic(t)), cex = 1.2), line = 2.75)
  if ( z.num == 1 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[1] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num == 2 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[2] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num %in% !c(1, 2) ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ? ' * .(sig)), cex = 1.2),
      line = 2.25)
  }
  plot(zph.log[var.num], ann = FALSE)
  ifelse ( zph.log$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('ln(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  if ( z.num == 1 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[1] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num == 2 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[2] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num %in% !c(1, 2) ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ? ' * .(sig)), cex = 1.2),
      line = 2.25)
  }
  plot(zph.km[var.num], ann = FALSE)
  ifelse ( zph.km$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('1-KM(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  if ( z.num == 1 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[1] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num == 2 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[2] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num %in% !c(1, 2) ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ? ' * .(sig)), cex = 1.2),
      line = 2.25)
  }
  plot(zph.rank[var.num], ann = FALSE)
  ifelse ( zph.rank$table[var.num, 3] < 0.05, sig <- '*', sig <- '' )
  title(xlab = list(bquote('Rank(' * italic(t) * ')'), cex = 1.2), line = 2.75)
  if ( z.num == 1 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[1] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num == 2 ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ' * italic(z)[2] * .(sig)),
      cex = 1.2), line = 2.25)
  }
  if ( z.num %in% !c(1, 2) ) {
    title(ylab = list(bquote(beta(italic(t)) * ' for ? ' * .(sig)), cex = 1.2),
      line = 2.25)
  }
  dev.off()
  #embed_fonts(paste(directory, '/', var.labs[i], '.pdf', sep = ''))
}
