################################################################################
################################################################################
####                                                                        ####
#### File 5 of replication code for:                                        ####
####   Park, Sunhee and David J. Hendry. "Reassessing Schoenfeld Residual   ####
####     Tests of Proportional Hazards in Political Science Event History   ####
####     Analyses." American Journal of Political Science.                  ####
####                                                                        ####
####      - This file creates Tables 2 and 3 from the main text and Tables  ####
####        SI.G.1 through SI.G.38 in the online Supporting Information and ####
####        saves them in the directory "tables", and Figures 2 and 3 from  ####
####        the main text and Figures SI.G.1 through SI.G.56 in the online  ####
####        Supporting Information and saves them in the directory          ####
####        "figures".                                                      ####
####        *** Note: The actual replication datasets are not provided      ####
####            because they are datasets from previously published work    ####
####            and the original authors and journals maintain sole rights  ####
####            to their distribution.  In all cases, the datasets referred ####
####            to herein were made publicly available by the original      ####
####            authors or were transmitted through personal                ####
####            correspondence.  Please contact Park and Hendry or the      ####
####            original authors for information about how to obtain them.  ####
####                                                                        ####
################################################################################
################################################################################

#set.wd(...)

library(foreign)
library(survival)
library(eha)
library(norm)
library(np)
# For Latex-style font in R graphical devices
#library(extrafont)
#loadfonts(device = 'pdf')
#Sys.setenv(R_GSCMD = 'INSERT GHOSTSCRIPT PATH NAME HERE')
#font_install('fontcm')

# To be used later
p.val.cols <- c(3, 6, 9, 12)
rho.cols <- c(1, 4, 7, 10)
chi.sq.cols <- c(2, 5, 8, 11)
cols <- c('t ', 'ln(t) ', '1-KM(t) ', 'Rank(t) ')

################################################################################
## Replications of published work
################################################################################


## Main Text


################################################################################
##
## Martin, Lanny W. 2004. "The Government Agenda in Parliamentary Democracies."
##   American Journal of Political Science 48(3): 445-461.
##
##     - results identical to original paper
##

martin2004 <- read.dta('data/nonpublic/martin2004/agenda6.dta')

martin2004$fail <- 1

surv <- with(martin2004, Surv(durat, fail))
martin2004.cox1 <- coxph(surv ~ cbcopsibw + wcabdivbw + opcopsibw + wopdivbw + 
  dim2 + dim3 + dim4 + dim5 + dim6 + dim7 + dim8 + germany + belgium + luxem, 
  method = 'breslow', data = martin2004)
summary(martin2004.cox1)

martin2004.ph.tab1 <- zphTable(martin2004.cox1, digits = 3)
martin2004.ph.tab1
martin2004.ph.tab1[, c(3, 6, 9, 12)]
martin2004.var.labs1 <- rownames(martin2004.ph.tab1)
martin2004.var.labs.pretty1 <- c('Government Issue Saliency', 
  'Government Issue Divisiveness', 'Opposition Issue Saliency', 
  'Opposition Issue Divisiveness', 'Foreign Policy', 'Industrial Policy',
  'Social Policy', 'Clerical Policy', 'Agricultural Policy', 'Regional Policy',
  'Environmental Policy', 'Germany', 'Belgium', 'Luxembourg')

star.tab.temp <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab1)), ncol = 4)

for ( i in 1:nrow(martin2004.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( martin2004.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
martin2004.ph.tab.pretty1 <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab1)),
  ncol = 4)
rownames(martin2004.ph.tab.pretty1) <- c(martin2004.var.labs.pretty1, 'GLOBAL')
colnames(martin2004.ph.tab.pretty1) <- cols
for ( i in 1:nrow(martin2004.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    martin2004.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", martin2004.ph.tab1[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  martin2004.ph.tab.pretty1[nrow(martin2004.ph.tab1), j] <-
    paste(sprintf("%.3f", martin2004.ph.tab1[nrow(martin2004.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp[nrow(martin2004.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-02.txt')
  cat('Table 2', '\n\n')
  as.data.frame(martin2004.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table 2 supplemental information', '\n\n')
  martin2004.ph.tab1
sink()

zphPlot1(cox.fit = martin2004.cox1, var.lab = 'cbcopsibw',
  var.lab.pretty = 'Government Issue Saliency', directory = 'figures',
  fig.name = 'figure-02')
################################################################################


################################################################################
##
## Bennett, D. Scott. 1997. "Testing Alternative Models of Alliance Duration,
##   1816-1984." American Journal of Political Science 41(3): 846-878.
##
## Replicated in:
##
## Box-Steffensmeier, Janet M., Dan Reiter, and Christopher Zorn. 2003.
##   "Nonproportional Hazards and Event History Analysis in International
##   Relations." Journal of Conflict Resolution 47(1): 33-53.
##
##     - cannot replicate original Weibull analyses
##         - likely due to differences between R and LIMDEP
##         - R results and Stata results (from Box-Steffensmeier et al.) match
##     - Cox results from Box-Steffensmeier et al. are identical
##

bennett1997 <- read.dta('data/nonpublic/bennett1997/bennett1997.dta')
bennett1997$t <- bennett1997$'_t'
bennett1997$t0 <- bennett1997$'_t0'

bennett1997.weib <- aftreg(Surv(t0, t, newcens) ~ avscch5 + avscahc5 + muthr5c + 
  sumcwch + symmet + concapc + avglib + avgsspol + numstate + wartime + 
  warendmx, param = 'lifeExp', dist = 'weibull', data = bennett1997)
summary(bennett1997.weib)

bennett1997.cox <- coxph(Surv(t0, t, newcens) ~ avscch5 + avscahc5 + muthr5c + 
  sumcwch + symmet + concapc + avglib + avgsspol + numstate + wartime + 
  warendmx + cluster(allynum), data = bennett1997, x = TRUE, method = "efron")
summary(bennett1997.cox)

bennett1997.ph.tab <- zphTable(bennett1997.cox, digits = 3)
bennett1997.ph.tab
bennett1997.ph.tab[, c(3, 6, 9, 12)]

bennett1997.var.labs <- rownames(bennett1997.ph.tab)
bennett1997.var.labs.pretty <- c('Change in Security', 
  'Alliance Security Improvement', 'Mutual Threat', 'Capability Change', 
  'Symmetry', 'Capability Concentration', 'Democracy (Liberal)', 
  'Polity Change', 'Number of States', 'Wartime', 'War Termination')

star.tab.temp <- matrix(rep(NA, 4 * nrow(bennett1997.ph.tab)), ncol = 4)

for ( i in 1:nrow(bennett1997.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( bennett1997.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
bennett1997.ph.tab.pretty <- matrix(rep(NA, 4 * nrow(bennett1997.ph.tab)),
  ncol = 4)
rownames(bennett1997.ph.tab.pretty) <- c(bennett1997.var.labs.pretty, 'GLOBAL')
colnames(bennett1997.ph.tab.pretty) <- cols
for ( i in 1:nrow(bennett1997.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    bennett1997.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", bennett1997.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  bennett1997.ph.tab.pretty[nrow(bennett1997.ph.tab), j] <-
    paste(sprintf("%.3f", bennett1997.ph.tab[nrow(bennett1997.ph.tab),
    chi.sq.cols[j]]), star.tab.temp[nrow(bennett1997.ph.tab), j], sep = '')
}

sink(file = 'tables/table-03.txt')
  cat('Table 3', '\n\n')
  as.data.frame(bennett1997.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table 3 supplemental information', '\n\n')
  bennett1997.ph.tab
sink()

zphPlot1(cox.fit = bennett1997.cox, var.lab = 'symmet',
  var.lab.pretty = 'Symmetry', directory = 'figures', fig.name = 'figure-03')
################################################################################


## Supporting Information and additional supplemental analyses


################################################################################
##
## Bennett, D. Scott. 1997. "Testing Alternative Models of Alliance Duration,
##   1816-1984." American Journal of Political Science 41(3): 846-878.
##
## Replicated in:
##
## Box-Steffensmeier, Janet M., Dan Reiter, and Christopher Zorn. 2003.
##   "Nonproportional Hazards and Event History Analysis in International
##   Relations." Journal of Conflict Resolution 47(1): 33-53.
##
##     - cannot replicate original Weibull analyses
##         - likely due to differences between R and LIMDEP
##         - R results and Stata results (from Box-Steffensmeier et al.) match
##     - Cox results from Box-Steffensmeier et al. are identical
##

bennett1997 <- read.dta('data/nonpublic/bennett1997/bennett1997.dta')
bennett1997$t <- bennett1997$'_t'
bennett1997$t0 <- bennett1997$'_t0'

bennett1997.weib <- aftreg(Surv(t0, t, newcens) ~ avscch5 + avscahc5 + muthr5c + 
  sumcwch + symmet + concapc + avglib + avgsspol + numstate + wartime + 
  warendmx, param = 'lifeExp', dist = 'weibull', data = bennett1997)
summary(bennett1997.weib)

bennett1997.cox <- coxph(Surv(t0, t, newcens) ~ avscch5 + avscahc5 + muthr5c + 
  sumcwch + symmet + concapc + avglib + avgsspol + numstate + wartime + 
  warendmx + cluster(allynum), data = bennett1997, x = TRUE, method = "efron")
summary(bennett1997.cox)

bennett1997.ph.tab <- zphTable(bennett1997.cox, digits = 3)
bennett1997.ph.tab
bennett1997.ph.tab[, c(3, 6, 9, 12)]

bennett1997.var.labs <- rownames(bennett1997.ph.tab)
bennett1997.var.labs.pretty <- c('Change in Security', 
  'Alliance Security Improvement', 'Mutual Threat', 'Capability Change', 
  'Symmetry', 'Capability Concentration', 'Democracy (Liberal)', 
  'Polity Change', 'Number of States', 'Wartime', 'War Termination')

star.tab.temp <- matrix(rep(NA, 4 * nrow(bennett1997.ph.tab)), ncol = 4)

for ( i in 1:nrow(bennett1997.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( bennett1997.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
bennett1997.ph.tab.pretty <- matrix(rep(NA, 4 * nrow(bennett1997.ph.tab)),
  ncol = 4)
rownames(bennett1997.ph.tab.pretty) <- c(bennett1997.var.labs.pretty, 'GLOBAL')
colnames(bennett1997.ph.tab.pretty) <- cols
for ( i in 1:nrow(bennett1997.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    bennett1997.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", bennett1997.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  bennett1997.ph.tab.pretty[nrow(bennett1997.ph.tab), j] <-
    paste(sprintf("%.3f", bennett1997.ph.tab[nrow(bennett1997.ph.tab),
    chi.sq.cols[j]]), star.tab.temp[nrow(bennett1997.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g01.txt')
  cat('Table SI.G.1', '\n\n')
  as.data.frame(bennett1997.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.1 supplemental information', '\n\n')
  bennett1997.ph.tab
sink()

zphPlot1(cox.fit = bennett1997.cox, var.lab = 'muthr5c',
  var.lab.pretty = 'Mutual Threat', directory = 'figures',
  fig.name = 'figure-si-g01')
zphPlot1(cox.fit = bennett1997.cox, var.lab = 'symmet',
  var.lab.pretty = 'Symmetry', directory = 'figures',
  fig.name = 'figure-si-g02')
zphPlot1(cox.fit = bennett1997.cox, var.lab = 'warendmx',
  var.lab.pretty = 'War Termination', directory = 'figures',
  fig.name = 'figure-si-g03')

zphPlots(cox.fit = bennett1997.cox, var.labs = bennett1997.var.labs,
  var.labs.pretty = bennett1997.var.labs.pretty,
  directory = 'figures/supplemental/bennett1997')
################################################################################


################################################################################
##
## Bennett, D. Scott, and Allan C. Stam III. 1996. "The Duration of Interstate
##   Wars, 1816-1985." American Political Science Review 90(2): 239-257.
##
##     - cannot replicate Table 1, Models 2, 4, and 5 (models do not converge)
##     - Cox specifications for same models also fail to converge in R
##

bennettandstam1996.year <-
  read.csv('data/nonpublic/bennettandstam1996/one_per_year.csv')
bennettandstam1996.year$end <- 0
bennettandstam1996.year[
  bennettandstam1996.year$year == bennettandstam1996.year$lastyear, ]$end <- 1
bennettandstam1996.year$t0 <- 0
temp <- unique(bennettandstam1996.year$warnum)
for ( i in 1:length(temp) ) {
  temp2 <- which(bennettandstam1996.year$warnum == temp[i] &
    !is.na(bennettandstam1996.year$time))
  if ( length(temp2) > 1 ) {
    bennettandstam1996.year[temp2, ]$t0 <-
      c(0, embed(bennettandstam1996.year[temp2, ]$time, 2)[, 2])
  }
}
bennettandstam1996.war <-
  read.csv('data/nonpublic/bennettandstam1996/one_per_war.csv')

surv1 <- with(bennettandstam1996.year, Surv(t0, time, end))

bennettandstam1996.weib1 <- aftreg(surv1 ~ popratio + year + nactors, 
  dist = 'weibull', data = bennettandstam1996.year, param = 'lifeExp')
bennettandstam1996.weib3 <- aftreg(surv1 ~ reprsum + demosum, dist = 'weibull', 
  data = bennettandstam1996.year, param = 'lifeExp')  
summary(bennettandstam1996.weib1)
summary(bennettandstam1996.weib3)

bennettandstam1996.cox1 <- coxph(surv1 ~ popratio + year + nactors,
  data = bennettandstam1996.year)
bennettandstam1996.cox3 <- coxph(surv1 ~ reprsum + demosum,
  data = bennettandstam1996.year)
summary(bennettandstam1996.cox1)
summary(bennettandstam1996.cox3)

bennettandstam1996.ph.tab1 <- zphTable(bennettandstam1996.cox1, digits = 3)
bennettandstam1996.ph.tab2 <- zphTable(bennettandstam1996.cox3, digits = 3)
bennettandstam1996.ph.tab1
bennettandstam1996.ph.tab1[, c(3, 6, 9, 12)]
bennettandstam1996.ph.tab2
bennettandstam1996.ph.tab2[, c(3, 6, 9, 12)]

bennettandstam1996.var.labs1 <- rownames(bennettandstam1996.ph.tab1)
bennettandstam1996.var.labs.pretty1 <- c('Population Ratio', 'Year', 
  'No. of Actors')
bennettandstam1996.var.labs2 <- rownames(bennettandstam1996.ph.tab2)
bennettandstam1996.var.labs.pretty2 <- c('Repression', 'Democracy')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(bennettandstam1996.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(bennettandstam1996.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( bennettandstam1996.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
bennettandstam1996.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(bennettandstam1996.ph.tab1)), ncol = 4)
rownames(bennettandstam1996.ph.tab.pretty1) <-
  c(bennettandstam1996.var.labs.pretty1, 'GLOBAL')
colnames(bennettandstam1996.ph.tab.pretty1) <- cols
for ( i in 1:nrow(bennettandstam1996.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    bennettandstam1996.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", bennettandstam1996.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  bennettandstam1996.ph.tab.pretty1[nrow(bennettandstam1996.ph.tab1), j] <-
    paste(sprintf("%.3f", bennettandstam1996.ph.tab1[
      nrow(bennettandstam1996.ph.tab1), chi.sq.cols[j]]),
    star.tab.temp1[nrow(bennettandstam1996.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g02.txt')
  cat('Table SI.G.2', '\n\n')
  as.data.frame(bennettandstam1996.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.2 supplemental information', '\n\n')
  bennettandstam1996.ph.tab1
sink()

zphPlot1(cox.fit = bennettandstam1996.cox1, var.lab = 'year',
  var.lab.pretty = 'Year', directory = 'figures',
  fig.name = 'figure-si-g04')

zphPlots(cox.fit = bennettandstam1996.cox1, 
  var.labs = bennettandstam1996.var.labs1, 
  var.labs.pretty = bennettandstam1996.var.labs.pretty1,
  directory = 'figures/supplemental/bennettandstam1996/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(bennettandstam1996.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(bennettandstam1996.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( bennettandstam1996.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
bennettandstam1996.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(bennettandstam1996.ph.tab2)), ncol = 4)
rownames(bennettandstam1996.ph.tab.pretty2) <-
  c(bennettandstam1996.var.labs.pretty2, 'GLOBAL')
colnames(bennettandstam1996.ph.tab.pretty2) <- cols
for ( i in 1:nrow(bennettandstam1996.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    bennettandstam1996.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", bennettandstam1996.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  bennettandstam1996.ph.tab.pretty2[nrow(bennettandstam1996.ph.tab2), j] <-
    paste(sprintf("%.3f", bennettandstam1996.ph.tab2[
      nrow(bennettandstam1996.ph.tab2), chi.sq.cols[j]]),
    star.tab.temp2[nrow(bennettandstam1996.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g03.txt')
  cat('Table SI.G.3', '\n\n')
  as.data.frame(bennettandstam1996.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.3 supplemental information', '\n\n')
  bennettandstam1996.ph.tab2
sink()

zphPlot1(cox.fit = bennettandstam1996.cox3, var.lab = 'reprsum',
  var.lab.pretty = 'Repression', directory = 'figures',
  fig.name = 'figure-si-g05')
zphPlot1(cox.fit = bennettandstam1996.cox3, var.lab = 'demosum',
  var.lab.pretty = 'Democracy', directory = 'figures',
  fig.name = 'figure-si-g06')

zphPlots(cox.fit = bennettandstam1996.cox3,
  var.labs = bennettandstam1996.var.labs2,
  var.labs.pretty = bennettandstam1996.var.labs.pretty2,
  directory = 'figures/supplemental/bennettandstam1996/model3')
################################################################################


################################################################################
##
## Box-Steffensmeier, Janet M. 1996. "A Dynamic Analysis of The Role of War
##   Chests in Campaign Strategy." American Journal of Political Science 40(2):
##   352-371.
##
##     - results are identical to original paper
##

boxsteffensmeier1996 <-
  read.dta('data/nonpublic/boxsteffensmeier1996/warchest_updated.dta')
boxsteffensmeier1996$t0 <- 0
temp <- unique(boxsteffensmeier1996$caseid)
for ( i in 1:length(temp) ) {
  temp2 <- which(boxsteffensmeier1996$caseid == temp[i] &
    !is.na(boxsteffensmeier1996$te))
  if ( length(temp2) > 1 ) {
    boxsteffensmeier1996[temp2, ]$t0 <-
    c(0, embed(boxsteffensmeier1996[temp2, ]$te, 2)[, 2])
  }
}

surv <- with(boxsteffensmeier1996, Surv(t0, te, cut_hi))

boxsteffensmeier1996.cox <- coxph(surv ~ south + dem + iv + ec, 
  method = 'breslow', data = boxsteffensmeier1996)
summary(boxsteffensmeier1996.cox)

boxsteffensmeier1996.ph.tab <- zphTable(boxsteffensmeier1996.cox, digits = 3)
boxsteffensmeier1996.ph.tab
boxsteffensmeier1996.ph.tab[, c(3, 6, 9, 12)]

boxsteffensmeier1996.var.labs <- rownames(boxsteffensmeier1996.ph.tab)
boxsteffensmeier1996.var.labs.pretty <- c('South', 'Party', 'Prior Vote',
  'War Chest')

star.tab.temp <- matrix(rep(NA, 4 * nrow(boxsteffensmeier1996.ph.tab)),
  ncol = 4)

for ( i in 1:nrow(boxsteffensmeier1996.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( boxsteffensmeier1996.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
boxsteffensmeier1996.ph.tab.pretty <-
  matrix(rep(NA, 4 * nrow(boxsteffensmeier1996.ph.tab)), ncol = 4)
rownames(boxsteffensmeier1996.ph.tab.pretty) <-
  c(boxsteffensmeier1996.var.labs.pretty, 'GLOBAL')
colnames(boxsteffensmeier1996.ph.tab.pretty) <- cols
for ( i in 1:nrow(boxsteffensmeier1996.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    boxsteffensmeier1996.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", boxsteffensmeier1996.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  boxsteffensmeier1996.ph.tab.pretty[nrow(boxsteffensmeier1996.ph.tab), j] <-
    paste(sprintf("%.3f", boxsteffensmeier1996.ph.tab[
      nrow(boxsteffensmeier1996.ph.tab), chi.sq.cols[j]]),
    star.tab.temp[nrow(boxsteffensmeier1996.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g04.txt')
  cat('Table SI.G.4', '\n\n')
  as.data.frame(boxsteffensmeier1996.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.4 supplemental information', '\n\n')
  boxsteffensmeier1996.ph.tab
sink()

zphPlots(cox.fit = boxsteffensmeier1996.cox, 
  var.labs = boxsteffensmeier1996.var.labs,
  var.labs.pretty = boxsteffensmeier1996.var.labs.pretty,
  directory = 'figures/supplemental/boxsteffensmeier1996')
################################################################################


################################################################################
##
## Box-Steffensmeier, Janet M., Laura W. Arnold, and Christopher J. W. Zorn.
##   1997. "The Strategic Timing of Position Taking in Congress: A Study of the
##   North American Free Trade Agreement." American Political Science Review
##   91(2): 324-338.
##
##     - results are identical to original paper
##

boxsteffensmeieretal1997 <-
  read.dta('data/nonpublic/boxsteffensmeieretal1997/TIMING.DTA',
  convert.factors = FALSE)
boxsteffensmeieretal1997$censor <- 1
  
surv <- with(boxsteffensmeieretal1997, Surv(timing, censor))

boxsteffensmeieretal1997.cox <- coxph(surv ~ pscenter + pecenter + perotsq +
  mexbordr + hhcenter + corptpct + labtpct + ncomact + rleader + dleader + 
  inter1 + inter2, data = boxsteffensmeieretal1997, method = 'breslow')
summary(boxsteffensmeieretal1997.cox)

boxsteffensmeieretal1997.ph.tab <- zphTable(boxsteffensmeieretal1997.cox, 
  digits = 3)
boxsteffensmeieretal1997.ph.tab
boxsteffensmeieretal1997.ph.tab[, c(3, 6, 9, 12)]

boxsteffensmeieretal1997.var.labs <- rownames(boxsteffensmeieretal1997.ph.tab)
boxsteffensmeieretal1997.var.labs.pretty <- c('Union Membership', 
  'Perot Vote Pct.', 'Perot Vote Pct. Sq.', 'Mexican Border', 'HH Income',
  'Corp. Contributions', 'Labor Contributions', 'NAFTA Cmte.', 
  'Rep. Leadership', 'Dem. Leadership', 'Ideology X Union', 
  'Ideology X HH Income')

star.tab.temp <- matrix(rep(NA, 4 * nrow(boxsteffensmeieretal1997.ph.tab)),
  ncol = 4)

for ( i in 1:nrow(boxsteffensmeieretal1997.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( boxsteffensmeieretal1997.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
boxsteffensmeieretal1997.ph.tab.pretty <-
  matrix(rep(NA, 4 * nrow(boxsteffensmeieretal1997.ph.tab)), ncol = 4)
rownames(boxsteffensmeieretal1997.ph.tab.pretty) <-
  c(boxsteffensmeieretal1997.var.labs.pretty, 'GLOBAL')
colnames(boxsteffensmeieretal1997.ph.tab.pretty) <- cols
for ( i in 1:nrow(boxsteffensmeieretal1997.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    boxsteffensmeieretal1997.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", boxsteffensmeieretal1997.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  boxsteffensmeieretal1997.ph.tab.pretty[
    nrow(boxsteffensmeieretal1997.ph.tab), j] <-
      paste(sprintf("%.3f", boxsteffensmeieretal1997.ph.tab[
        nrow(boxsteffensmeieretal1997.ph.tab), chi.sq.cols[j]]),
      star.tab.temp[nrow(boxsteffensmeieretal1997.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g05.txt')
  cat('Table SI.G.5', '\n\n')
  as.data.frame(boxsteffensmeieretal1997.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.5 supplemental information', '\n\n')
  boxsteffensmeieretal1997.ph.tab
sink()

zphPlot1(cox.fit = boxsteffensmeieretal1997.cox, var.lab = 'mexbordr',
  var.lab.pretty = 'Mexican Border', directory = 'figures',
  fig.name = 'figure-si-g07')

zphPlots(cox.fit = boxsteffensmeieretal1997.cox, 
  var.labs = boxsteffensmeieretal1997.var.labs,
  var.labs.pretty = boxsteffensmeieretal1997.var.labs.pretty,
  directory = 'figures/supplemental/boxsteffensmeieretal1997')
################################################################################


################################################################################
##
## Chiozza, Giacomo, and H. E. Goemans. 2004. "International Conflict and the 
##   Tenure of Leaders: Is War Still 'Ex Post' Inefficient?" American Journal of
##   Political Science 48(3): 604-619.
##
##     - results are identical to original paper
##     - only pre-presentation models estimated
##

chiozzaandgoemans2004 <-
  read.dta('data/nonpublic/chiozzaandgoemans2004/toR.dta',
  convert.dates = FALSE)

chiozzaandgoemans2004$txmixed <- with(chiozzaandgoemans2004, log(t) * mixed)
chiozzaandgoemans2004$txdemparl <- with(chiozzaandgoemans2004, log(t) * demparl)
chiozzaandgoemans2004$txdempres <- with(chiozzaandgoemans2004, log(t) * dempres)
chiozzaandgoemans2004$txcivwar <- with(chiozzaandgoemans2004, log(t) * civwar)
chiozzaandgoemans2004$txlnencap <- with(chiozzaandgoemans2004, log(t) * lnencap)
chiozzaandgoemans2004$txlntropen <-
  with(chiozzaandgoemans2004, log(t) * lntropen)
chiozzaandgoemans2004$txlnpop <- with(chiozzaandgoemans2004, log(t) * lnpop)
chiozzaandgoemans2004$txage <- with(chiozzaandgoemans2004, log(t) * age)
chiozzaandgoemans2004$txiniwar <- with(chiozzaandgoemans2004, log(t) * iniwar)

L <- as.matrix(chiozzaandgoemans2004[, c('t', 't0', 'd', 'mixed', 'txmixed',
  'demparl', 'txdemparl', 'dempres', 'txdempres', 'civwar', 'txcivwar', 
  'lnencap', 'txlnencap','encapchg', 'lntropen', 'tropenchg', 'lnpop', 
  'txlnpop', 'age', 'txage', 'powtimes', 'inish', 'tginvsh', 'dwinsh',
  'dlosesh', 'ddrawsh', 'iniwar', 'txiniwar', 'tginvwar', 'dwinwar', 'dlosewar',
  'ddrawwar')])

s <- prelim.norm(L)
thetahat.0 <- em.norm(s, criterion = 1e-09)

thetahat <- thetahat.0
thetahat <- getparam.norm(s, thetahat)

set.seed(1510)
trk <- seq(1, nrow(L), 1)
boot.1 <- sample(trk, 1 + abs(nrow(L) / 4), replace = TRUE)

s.1 <- prelim.norm(L[boot.1, ])
thetahat.1 <- em.norm(s.1, criterion = 1e-09)
thetahat.1 <- getparam.norm(s.1, thetahat.1)

thetahat$mu <- thetahat.1$mu
thetahat$sigma <- thetahat.1$sigma

thetahat <- makeparam.norm(s, thetahat)

rngseed(1510)
theta.1 <- da.norm(s, thetahat, steps = 500, showits = TRUE)
L.1 <- as.data.frame(imp.norm(s, theta.1, L))

L.1$ccode <- chiozzaandgoemans2004$ccode

#model1.tx <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + lnencap + txlnencap + encapchg + lntropen + 
#  tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + tginvsh + 
#  dwinsh + dlosesh + ddrawsh + iniwar + tginvwar + dwinwar + dlosewar + 
#  ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = chiozzaandgoemans2004, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

model1.L1 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh  + dwinsh + dlosesh + ddrawsh  + iniwar + tginvwar + dwinwar + 
  dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, data = L.1,
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model1.L1.id <- cox.zph(model1.L1, transform = 'identity')  
fit.model1.L1.log <- cox.zph(model1.L1, transform = 'log')
fit.model1.L1.km <- cox.zph(model1.L1, transform = 'km')
fit.model1.L1.rank <- cox.zph(model1.L1, transform = 'rank')

#model1.tx.L1 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + txiniwar + tginvwar + 
#  dwinwar + dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = L.1, 
#  control = coxph.control(eps = 1e-09, iter.max = 1000, outer.max = 100))

#m1.coef.L1 <- list(coef(model1.tx.L1))
#m1.se.L1 <- list(sqrt(diag(vcov(model1.tx.L1))))

chiozzaandgoemans2004$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
chiozzaandgoemans2004$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
chiozzaandgoemans2004$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
chiozzaandgoemans2004$dparldwinsh <-
  with(chiozzaandgoemans2004, demparl * dwinsh)
chiozzaandgoemans2004$dparldlosesh <-
  with(chiozzaandgoemans2004, demparl * dlosesh)
chiozzaandgoemans2004$dparlddrawsh <-
  with(chiozzaandgoemans2004, demparl * ddrawsh)
chiozzaandgoemans2004$dpresdwinsh <-
  with(chiozzaandgoemans2004, dempres * dwinsh)
chiozzaandgoemans2004$dpresdlosesh <-
  with(chiozzaandgoemans2004, dempres * dlosesh)
chiozzaandgoemans2004$dpresddrawsh <-
  with(chiozzaandgoemans2004, dempres * ddrawsh)
chiozzaandgoemans2004$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
chiozzaandgoemans2004$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
chiozzaandgoemans2004$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
chiozzaandgoemans2004$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
chiozzaandgoemans2004$mixdlosewar <-
  with(chiozzaandgoemans2004, mixed * dlosewar)
chiozzaandgoemans2004$mixddrawwar <-
  with(chiozzaandgoemans2004, mixed * ddrawwar)
chiozzaandgoemans2004$dparldwinwar <-
  with(chiozzaandgoemans2004, demparl * dwinwar)
chiozzaandgoemans2004$dparldlosewar <-
  with(chiozzaandgoemans2004, demparl * dlosewar)
chiozzaandgoemans2004$dparlddrawwar <-
  with(chiozzaandgoemans2004, demparl * ddrawwar)
chiozzaandgoemans2004$dpresdwinwar <-
  with(chiozzaandgoemans2004, dempres * dwinwar)
chiozzaandgoemans2004$dpresdlosewar <-
  with(chiozzaandgoemans2004, dempres * dlosewar)
chiozzaandgoemans2004$dpresddrawwar <-
  with(chiozzaandgoemans2004, dempres * ddrawwar)
chiozzaandgoemans2004$autdwinwar <- with(chiozzaandgoemans2004, autoc * dwinwar)
chiozzaandgoemans2004$autdlosewar <-
  with(chiozzaandgoemans2004, autoc * dlosewar)
chiozzaandgoemans2004$autddrawwar <-
  with(chiozzaandgoemans2004, autoc * ddrawwar)

L.1$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
L.1$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
L.1$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
L.1$dparldwinsh <- with(chiozzaandgoemans2004, demparl * dwinsh)
L.1$dparldlosesh <- with(chiozzaandgoemans2004, demparl * dlosesh)
L.1$dparlddrawsh <- with(chiozzaandgoemans2004, demparl * ddrawsh)
L.1$dpresdwinsh <- with(chiozzaandgoemans2004, dempres * dwinsh)
L.1$dpresdlosesh <- with(chiozzaandgoemans2004, dempres * dlosesh)
L.1$dpresddrawsh <- with(chiozzaandgoemans2004, dempres * ddrawsh)
L.1$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
L.1$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
L.1$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
L.1$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
L.1$mixdlosewar <- with(chiozzaandgoemans2004, mixed * dlosewar)
L.1$mixddrawwar <- with(chiozzaandgoemans2004, mixed * ddrawwar)
L.1$dparldwinwar <- with(chiozzaandgoemans2004, demparl * dwinwar)
L.1$dparldlosewar <- with(chiozzaandgoemans2004, demparl * dlosewar)
L.1$dparlddrawwar <- with(chiozzaandgoemans2004, demparl * ddrawwar)
L.1$dpresdwinwar <- with(chiozzaandgoemans2004, dempres*dwinwar)
L.1$dpresdlosewar <- with(chiozzaandgoemans2004, dempres*dlosewar)
L.1$dpresddrawwar <- with(chiozzaandgoemans2004, dempres*ddrawwar)
L.1$autdwinwar <- with(chiozzaandgoemans2004, autoc*dwinwar)
L.1$autdlosewar <- with(chiozzaandgoemans2004, autoc*dlosewar)
L.1$autddrawwar <- with(chiozzaandgoemans2004, autoc*ddrawwar)

#model2 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + lnencap + 
#  encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + tginvsh + 
#  autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + mixddrawsh + 
#  dparldwinsh + dparldlosesh + dparlddrawsh  + dpresdwinsh + dpresdlosesh + 
#  dpresddrawsh + iniwar + tginvwar  + autdwinwar + autdlosewar + autddrawwar + 
#  mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + dparldlosewar + 
#  dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar + frailty(ccode), 
#  na.action = na.exclude, data = chiozzaandgoemans2004, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#fit.model2 <- cox.zph(model2, transform = 'log')

#model2.tx <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + lnencap + txlnencap + encapchg + lntropen + 
#  tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + tginvsh + 
#  autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + mixddrawsh + 
#  dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + dpresdlosesh + 
#  dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + autddrawwar + 
#  mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + dparldlosewar + 
#  dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar + frailty(ccode), 
#  na.action = na.exclude, data = chiozzaandgoemans2004,
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

model2.L1 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg  + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
  dpresdlosesh + dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + 
  autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + 
  dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.1,
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model2.L1.id <- cox.zph(model2.L1, transform = 'identity')
fit.model2.L1.log <- cox.zph(model2.L1, transform = 'log')
fit.model2.L1.km <- cox.zph(model2.L1, transform = 'km')
fit.model2.L1.rank <- cox.zph(model2.L1, transform = 'rank')

#model2.tx.L1 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
#  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
#  dpresdlosesh + dpresddrawsh + iniwar +txiniwar + tginvwar + autdwinwar + 
#  autdlosewar + autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + 
#  dparldwinwar + dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
#  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.1,
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m2.coef.L1 <- list(coef(model2.tx.L1))
#m2.se.L1 <- list(sqrt(diag(vcov(model2.tx.L1))))

#Ht.L1 <- basehaz(model2.tx.L1, centered = FALSE)

thetahat <- thetahat.0
thetahat <- getparam.norm(s, thetahat)

set.seed(1015)
trk <- seq(1, nrow(L), 1)
boot.2 <- sample(trk, 1 + abs(nrow(L) / 4), replace = TRUE)

s.2 <- prelim.norm(L[boot.2, ])
thetahat.2 <- em.norm(s.2, criterion = 1e-09)
thetahat.2 <- getparam.norm(s.2, thetahat.2)

thetahat$mu <- thetahat.2$mu
thetahat$sigma <- thetahat.2$sigma

thetahat <- makeparam.norm(s, thetahat)

rngseed(2003)
theta.2 <- da.norm(s, thetahat, steps = 500, showits = TRUE)
L.2 <- as.data.frame(imp.norm(s, theta.2, L))

L.2$ccode <- chiozzaandgoemans2004$ccode

model1.L2 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + tginvwar + dwinwar + 
  dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, data = L.2, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model1.L2.id <- cox.zph(model1.L2, transform = 'identity')
fit.model1.L2.log <- cox.zph(model1.L2, transform = 'log')
fit.model1.L2.km <- cox.zph(model1.L2, transform = 'km')
fit.model1.L2.rank <- cox.zph(model1.L2, transform = 'rank')

#model1.tx.L2 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + txiniwar + tginvwar + 
#  dwinwar + dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = L.2, 
#  control = coxph.control(eps = 1e-09, iter.max = 100,outer.max=100))

#m1.coef.L2 <- list(coef(model1.tx.L2))
#m1.se.L2 <- list(sqrt(diag(vcov(model1.tx.L2))))

L.2$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
L.2$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
L.2$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
L.2$dparldwinsh <- with(chiozzaandgoemans2004, demparl * dwinsh)
L.2$dparldlosesh <- with(chiozzaandgoemans2004, demparl * dlosesh)
L.2$dparlddrawsh <- with(chiozzaandgoemans2004, demparl * ddrawsh)
L.2$dpresdwinsh <- with(chiozzaandgoemans2004, dempres * dwinsh)
L.2$dpresdlosesh <- with(chiozzaandgoemans2004, dempres * dlosesh)
L.2$dpresddrawsh <- with(chiozzaandgoemans2004, dempres * ddrawsh)
L.2$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
L.2$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
L.2$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
L.2$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
L.2$mixdlosewar <- with(chiozzaandgoemans2004, mixed * dlosewar)
L.2$mixddrawwar <- with(chiozzaandgoemans2004, mixed * ddrawwar)
L.2$dparldwinwar <- with(chiozzaandgoemans2004, demparl * dwinwar)
L.2$dparldlosewar <- with(chiozzaandgoemans2004, demparl * dlosewar)
L.2$dparlddrawwar <- with(chiozzaandgoemans2004, demparl * ddrawwar)
L.2$dpresdwinwar <- with(chiozzaandgoemans2004, dempres * dwinwar)
L.2$dpresdlosewar <- with(chiozzaandgoemans2004, dempres * dlosewar)
L.2$dpresddrawwar <- with(chiozzaandgoemans2004, dempres * ddrawwar)
L.2$autdwinwar <- with(chiozzaandgoemans2004, autoc * dwinwar)
L.2$autdlosewar <- with(chiozzaandgoemans2004, autoc * dlosewar)
L.2$autddrawwar <- with(chiozzaandgoemans2004, autoc * ddrawwar)

model2.L2 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
  dpresdlosesh + dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + 
  autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + 
  dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar + 
  frailty(ccode), na.action = na.exclude, data = L.2, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model2.L2.id <- cox.zph(model2.L2, transform = 'identity')
fit.model2.L2.log <- cox.zph(model2.L2, transform = 'log')
fit.model2.L2.km <- cox.zph(model2.L2, transform = 'km')
fit.model2.L2.rank <- cox.zph(model2.L2, transform = 'rank')

#model2.tx.L2 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
#  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
#  dpresdlosesh + dpresddrawsh + iniwar + txiniwar + tginvwar + autdwinwar + 
#  autdlosewar + autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + 
#  dparldwinwar + dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
#  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.2, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m2.coef.L2 <- list(coef(model2.tx.L2))
#m2.se.L2 <- list(sqrt(diag(vcov(model2.tx.L2))))

#Ht.L2 <- basehaz(model2.tx.L2, centered = FALSE)

thetahat <- thetahat.0
thetahat <- getparam.norm(s, thetahat)

set.seed(2107)
trk <- seq(1, nrow(L), 1)
boot.3 <- sample(trk, 1 + abs(nrow(L) / 4), replace = TRUE)

s.3 <- prelim.norm(L[boot.3, ])
thetahat.3 <- em.norm(s.3, criterion = 1e-09)
thetahat.3 <- getparam.norm(s.3, thetahat.3)

thetahat$mu <- thetahat.3$mu
thetahat$sigma <- thetahat.3$sigma

thetahat <- makeparam.norm(s, thetahat)

rngseed(3904)
theta.3 <- da.norm(s, thetahat, steps = 500, showits = TRUE)
L.3 <- as.data.frame(imp.norm(s, theta.3, L))

L.3$ccode <- chiozzaandgoemans2004$ccode

model1.L3 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + tginvwar + dwinwar + 
  dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, data = L.3, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model1.L3.id <- cox.zph(model1.L3, transform = 'identity')
fit.model1.L3.log <- cox.zph(model1.L3, transform = 'log')
fit.model1.L3.km <- cox.zph(model1.L3, transform = 'km')
fit.model1.L3.rank <- cox.zph(model1.L3, transform = 'rank')

#model1.tx.L3 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + txiniwar + tginvwar + 
#  dwinwar + dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = L.3, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m1.coef.L3 <- list(coef(model1.tx.L3))
#m1.se.L3 <- list(sqrt(diag(vcov(model1.tx.L3))))

L.3$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
L.3$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
L.3$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
L.3$dparldwinsh <- with(chiozzaandgoemans2004, demparl * dwinsh)
L.3$dparldlosesh <- with(chiozzaandgoemans2004, demparl * dlosesh)
L.3$dparlddrawsh <- with(chiozzaandgoemans2004, demparl * ddrawsh)
L.3$dpresdwinsh <- with(chiozzaandgoemans2004, dempres * dwinsh)
L.3$dpresdlosesh <- with(chiozzaandgoemans2004, dempres * dlosesh)
L.3$dpresddrawsh <- with(chiozzaandgoemans2004, dempres * ddrawsh)
L.3$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
L.3$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
L.3$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
L.3$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
L.3$mixdlosewar <- with(chiozzaandgoemans2004, mixed * dlosewar)
L.3$mixddrawwar <- with(chiozzaandgoemans2004, mixed * ddrawwar)
L.3$dparldwinwar <- with(chiozzaandgoemans2004, demparl * dwinwar)
L.3$dparldlosewar <- with(chiozzaandgoemans2004, demparl * dlosewar)
L.3$dparlddrawwar <- with(chiozzaandgoemans2004, demparl * ddrawwar)
L.3$dpresdwinwar <- with(chiozzaandgoemans2004, dempres * dwinwar)
L.3$dpresdlosewar <- with(chiozzaandgoemans2004, dempres * dlosewar)
L.3$dpresddrawwar <- with(chiozzaandgoemans2004, dempres * ddrawwar)
L.3$autdwinwar <- with(chiozzaandgoemans2004, autoc * dwinwar)
L.3$autdlosewar <- with(chiozzaandgoemans2004, autoc * dlosewar)
L.3$autddrawwar <- with(chiozzaandgoemans2004, autoc * ddrawwar)

model2.L3 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
  dpresdlosesh + dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + 
  autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + 
  dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar +
  frailty(ccode), na.action = na.exclude, data = L.3, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model2.L3.id <- cox.zph(model2.L3, transform = 'identity')
fit.model2.L3.log <- cox.zph(model2.L3, transform = 'log')
fit.model2.L3.km <- cox.zph(model2.L3, transform = 'km')
fit.model2.L3.rank <- cox.zph(model2.L3, transform = 'rank')

#model2.tx.L3 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
#  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
#  dpresdlosesh + dpresddrawsh + iniwar + txiniwar + tginvwar + autdwinwar + 
#  autdlosewar + autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + 
#  dparldwinwar + dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
#  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.3, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m2.coef.L3 <- list(coef(model2.tx.L3))
#m2.se.L3 <- list(sqrt(diag(vcov(model2.tx.L3))))

#Ht.L3 <- basehaz(model2.tx.L3, centered = FALSE)

thetahat <- thetahat.0
thetahat <- getparam.norm(s, thetahat)

set.seed(2605)
trk <- seq(1, nrow(L), 1)
boot.4 <- sample(trk, 1 + abs(nrow(L) / 4), replace = TRUE)

s.4 <- prelim.norm(L[boot.4, ])
thetahat.4 <- em.norm(s.4, criterion = 1e-09)
thetahat.4 <- getparam.norm(s.4, thetahat.4)

thetahat$mu <- thetahat.4$mu
thetahat$sigma <- thetahat.4$sigma

thetahat <- makeparam.norm(s, thetahat)

rngseed(1511)
theta.4 <- da.norm(s, thetahat, steps = 500, showits = TRUE)
L.4 <- as.data.frame(imp.norm(s, theta.4, L))

L.4$ccode <- chiozzaandgoemans2004$ccode

model1.L4 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + tginvwar + dwinwar + 
  dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, data = L.4,
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model1.L4.id <- cox.zph(model1.L4, transform = 'identity')
fit.model1.L4.log <- cox.zph(model1.L4, transform = 'log')
fit.model1.L4.km <- cox.zph(model1.L4, transform = 'km')
fit.model1.L4.rank <- cox.zph(model1.L4, transform = 'rank')

#model1.tx.L4 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar +txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + txiniwar + tginvwar + 
#  dwinwar + dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = L.4, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m1.coef.L4 <- list(coef(model1.tx.L4))
#m1.se.L4 <- list(sqrt(diag(vcov(model1.tx.L4))))

L.4$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
L.4$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
L.4$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
L.4$dparldwinsh <- with(chiozzaandgoemans2004, demparl * dwinsh)
L.4$dparldlosesh <- with(chiozzaandgoemans2004, demparl * dlosesh)
L.4$dparlddrawsh <- with(chiozzaandgoemans2004, demparl * ddrawsh)
L.4$dpresdwinsh <- with(chiozzaandgoemans2004, dempres * dwinsh)
L.4$dpresdlosesh <- with(chiozzaandgoemans2004, dempres * dlosesh)
L.4$dpresddrawsh <- with(chiozzaandgoemans2004, dempres * ddrawsh)
L.4$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
L.4$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
L.4$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
L.4$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
L.4$mixdlosewar <- with(chiozzaandgoemans2004, mixed * dlosewar)
L.4$mixddrawwar <- with(chiozzaandgoemans2004, mixed * ddrawwar)
L.4$dparldwinwar <- with(chiozzaandgoemans2004, demparl * dwinwar)
L.4$dparldlosewar <- with(chiozzaandgoemans2004, demparl * dlosewar)
L.4$dparlddrawwar <- with(chiozzaandgoemans2004, demparl * ddrawwar)
L.4$dpresdwinwar <- with(chiozzaandgoemans2004, dempres * dwinwar)
L.4$dpresdlosewar <- with(chiozzaandgoemans2004, dempres * dlosewar)
L.4$dpresddrawwar <- with(chiozzaandgoemans2004, dempres * ddrawwar)
L.4$autdwinwar <- with(chiozzaandgoemans2004, autoc * dwinwar)
L.4$autdlosewar <- with(chiozzaandgoemans2004, autoc * dlosewar)
L.4$autddrawwar <- with(chiozzaandgoemans2004, autoc * ddrawwar)

model2.L4 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
  dpresdlosesh + dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + 
  autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + 
  dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar + 
  frailty(ccode), na.action = na.exclude, data = L.4, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model2.L4.id <- cox.zph(model2.L4, transform = 'identity')
fit.model2.L4.log <- cox.zph(model2.L4, transform = 'log')
fit.model2.L4.km <- cox.zph(model2.L4, transform = 'km')
fit.model2.L4.rank <- cox.zph(model2.L4, transform = 'rank')

#model2.tx.L4 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
#  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
#  dpresdlosesh + dpresddrawsh + iniwar + txiniwar + tginvwar + autdwinwar + 
#  autdlosewar + autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + 
#  dparldwinwar + dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
#  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.4,
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m2.coef.L4 <- list(coef(model2.tx.L4))
#m2.se.L4 <- list(sqrt(diag(vcov(model2.tx.L4))))

#Ht.L4 <- basehaz(model2.tx.L4, centered = FALSE)

thetahat <- thetahat.0
thetahat <- getparam.norm(s, thetahat)

set.seed(2910)
trk <- seq(1, nrow(L), 1)
boot.5 <- sample(trk, 1 + abs(nrow(L) / 4), replace = TRUE)

s.5 <- prelim.norm(L[boot.5, ])
thetahat.5 <- em.norm(s.5, criterion = 1e-09)
thetahat.5 <- getparam.norm(s.5, thetahat.5)

thetahat$mu <- thetahat.5$mu
thetahat$sigma <- thetahat.5$sigma

thetahat <- makeparam.norm(s,thetahat)

rngseed(1029)
theta.5 <- da.norm(s, thetahat, steps = 500, showits = TRUE)
L.5 <- as.data.frame(imp.norm(s, theta.5, L))

L.5$ccode <- chiozzaandgoemans2004$ccode

model1.L5 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + tginvwar + dwinwar + 
  dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, data = L.5, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model1.L5.id <- cox.zph(model1.L5, transform = 'identity')
fit.model1.L5.log <- cox.zph(model1.L5, transform = 'log')
fit.model1.L5.km <- cox.zph(model1.L5, transform = 'km')
fit.model1.L5.rank <- cox.zph(model1.L5, transform = 'rank')

#model1.tx.L5 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + dwinsh + dlosesh + ddrawsh + iniwar + txiniwar + tginvwar + 
#  dwinwar + dlosewar + ddrawwar + frailty(ccode), na.action = na.exclude, 
#  data = L.5, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m1.coef.L5 <- list(coef(model1.tx.L5))
#m1.se.L5 <- list(sqrt(diag(vcov(model1.tx.L5))))

L.5$mixdwinsh <- with(chiozzaandgoemans2004, mixed * dwinsh)
L.5$mixdlosesh <- with(chiozzaandgoemans2004, mixed * dlosesh)
L.5$mixddrawsh <- with(chiozzaandgoemans2004, mixed * ddrawsh)
L.5$dparldwinsh <- with(chiozzaandgoemans2004, demparl * dwinsh)
L.5$dparldlosesh <- with(chiozzaandgoemans2004, demparl * dlosesh)
L.5$dparlddrawsh <- with(chiozzaandgoemans2004, demparl * ddrawsh)
L.5$dpresdwinsh <- with(chiozzaandgoemans2004, dempres * dwinsh)
L.5$dpresdlosesh <- with(chiozzaandgoemans2004, dempres * dlosesh)
L.5$dpresddrawsh <- with(chiozzaandgoemans2004, dempres * ddrawsh)
L.5$autdwinsh <- with(chiozzaandgoemans2004, autoc * dwinsh)
L.5$autdlosesh <- with(chiozzaandgoemans2004, autoc * dlosesh)
L.5$autddrawsh <- with(chiozzaandgoemans2004, autoc * ddrawsh)
L.5$mixdwinwar <- with(chiozzaandgoemans2004, mixed * dwinwar)
L.5$mixdlosewar <- with(chiozzaandgoemans2004, mixed * dlosewar)
L.5$mixddrawwar <- with(chiozzaandgoemans2004, mixed * ddrawwar)
L.5$dparldwinwar <- with(chiozzaandgoemans2004, demparl * dwinwar)
L.5$dparldlosewar <- with(chiozzaandgoemans2004, demparl * dlosewar)
L.5$dparlddrawwar <- with(chiozzaandgoemans2004, demparl * ddrawwar)
L.5$dpresdwinwar <- with(chiozzaandgoemans2004, dempres * dwinwar)
L.5$dpresdlosewar <- with(chiozzaandgoemans2004, dempres * dlosewar)
L.5$dpresddrawwar <- with(chiozzaandgoemans2004, dempres * ddrawwar)
L.5$autdwinwar <- with(chiozzaandgoemans2004, autoc * dwinwar)
L.5$autdlosewar <- with(chiozzaandgoemans2004, autoc * dlosewar)
L.5$autddrawwar <- with(chiozzaandgoemans2004, autoc * ddrawwar)

model2.L5 <- coxph(Surv(t0, t, d) ~ mixed + demparl + dempres + civwar + 
  lnencap + encapchg + lntropen + tropenchg + lnpop + age + powtimes + inish + 
  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
  dpresdlosesh + dpresddrawsh + iniwar + tginvwar + autdwinwar + autdlosewar + 
  autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + dparldwinwar + 
  dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + dpresddrawwar + 
  frailty(ccode), na.action = na.exclude, data = L.5, 
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

fit.model2.L5.id <- cox.zph(model2.L5, transform = 'identity')
fit.model2.L5.log <- cox.zph(model2.L5, transform = 'log')
fit.model2.L5.km <- cox.zph(model2.L5, transform = 'km')
fit.model2.L5.rank <- cox.zph(model2.L5, transform = 'rank')

#model2.tx.L5 <- coxph(Surv(t0, t, d) ~ mixed + txmixed + demparl + txdemparl + 
#  dempres + txdempres + civwar + txcivwar + lnencap + txlnencap + encapchg + 
#  lntropen + tropenchg + lnpop + txlnpop + age + txage + powtimes + inish + 
#  tginvsh + autdwinsh + autdlosesh + autddrawsh + mixdwinsh + mixdlosesh + 
#  mixddrawsh + dparldwinsh + dparldlosesh + dparlddrawsh + dpresdwinsh + 
#  dpresdlosesh + dpresddrawsh + iniwar + txiniwar + tginvwar + autdwinwar + 
#  autdlosewar + autddrawwar + mixdwinwar + mixdlosewar + mixddrawwar + 
#  dparldwinwar + dparldlosewar + dparlddrawwar + dpresdwinwar + dpresdlosewar + 
#  dpresddrawwar + frailty(ccode), na.action = na.exclude, data = L.5, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#m2.coef.L5 <- list(coef(model2.tx.L5))
#m2.se.L5 <- list(sqrt(diag(vcov(model2.tx.L5))))

#Ht.L5 <- basehaz(model2.tx.L5, centered = FALSE)

M <- 5

m1.rho.id <- cbind(fit.model1.L1.id$table[, 1], fit.model1.L2.id$table[, 1], 
  fit.model1.L3.id$table[, 1], fit.model1.L4.id$table[, 1], 
  fit.model1.L5.id$table[, 1])
m1.rho.log <- cbind(fit.model1.L1.log$table[, 1], fit.model1.L2.log$table[, 1], 
  fit.model1.L3.log$table[, 1], fit.model1.L4.log$table[, 1], 
  fit.model1.L5.log$table[, 1])
m1.rho.km <- cbind(fit.model1.L1.km$table[, 1], fit.model1.L2.km$table[, 1], 
  fit.model1.L3.km$table[, 1], fit.model1.L4.km$table[, 1], 
  fit.model1.L5.km$table[, 1])
m1.rho.rank <- cbind(fit.model1.L1.rank$table[, 1], 
  fit.model1.L2.rank$table[, 1], fit.model1.L3.rank$table[, 1], 
  fit.model1.L4.rank$table[, 1], fit.model1.L5.rank$table[, 1])

m1.rho.chi.id <- cbind(fit.model1.L1.id$table[, 2], fit.model1.L2.id$table[, 2],
  fit.model1.L3.id$table[, 2], fit.model1.L4.id$table[, 2], 
  fit.model1.L5.id$table[, 2])
m1.rho.chi.log <- cbind(fit.model1.L1.log$table[, 2], 
  fit.model1.L2.log$table[, 2], fit.model1.L3.log$table[, 2], 
  fit.model1.L4.log$table[, 2], fit.model1.L5.log$table[, 2])
m1.rho.chi.km <- cbind(fit.model1.L1.km$table[, 2], fit.model1.L2.km$table[, 2],
  fit.model1.L3.km$table[, 2], fit.model1.L4.km$table[, 2], 
  fit.model1.L5.km$table[, 2])
m1.rho.chi.rank <- cbind(fit.model1.L1.rank$table[, 2], 
  fit.model1.L2.rank$table[, 2], fit.model1.L3.rank$table[, 2], 
  fit.model1.L4.rank$table[, 2], fit.model1.L5.rank$table[, 2])

m1.rho.bar.id <- apply(m1.rho.id, 1, mean) 
m1.rho.bar.log <- apply(m1.rho.log, 1, mean) 
m1.rho.bar.km <- apply(m1.rho.km, 1, mean) 
m1.rho.bar.rank <- apply(m1.rho.rank, 1, mean) 

m1.d.bar.id <- apply(m1.rho.chi.id, 1, mean)
m1.d.bar.log <- apply(m1.rho.chi.log, 1, mean)
m1.d.bar.km <- apply(m1.rho.chi.km, 1, mean)
m1.d.bar.rank <- apply(m1.rho.chi.rank, 1, mean)

m1.s2.rho.id <- apply(sqrt(m1.rho.chi.id), 1, var)
m1.s2.rho.log <- apply(sqrt(m1.rho.chi.log), 1, var)
m1.s2.rho.km <- apply(sqrt(m1.rho.chi.km), 1, var)
m1.s2.rho.rank <- apply(sqrt(m1.rho.chi.rank), 1, var)

m1.D.tilde.id <- 
  (m1.d.bar.id - (1 - 1 / M) * m1.s2.rho.id) / (1 + (1 + 1 / M) * m1.s2.rho.id)
m1.D.tilde.log <- 
  (m1.d.bar.log - (1 - 1 / M) * m1.s2.rho.log) / 
  (1 + (1 + 1 / M) * m1.s2.rho.log)
m1.D.tilde.km <- 
  (m1.d.bar.km - (1 - 1 / M) * m1.s2.rho.km) / (1 + (1 + 1 / M) * m1.s2.rho.km)
m1.D.tilde.rank <- 
  (m1.d.bar.rank - (1 - 1 / M) * m1.s2.rho.rank) / 
  (1 + (1 + 1 / M) * m1.s2.rho.rank)

m1.b.rho.df.id <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m1.s2.rho.id)^(-1))^2
m1.b.rho.df.log <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m1.s2.rho.log)^(-1))^2
m1.b.rho.df.km <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m1.s2.rho.km)^(-1))^2
m1.b.rho.df.rank <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m1.s2.rho.rank)^(-1))^2

chiozzaandgoemans2004.ph.tab1 <- round(cbind('id.rho' = m1.rho.bar.id, 
  'id.D.tilde' = m1.D.tilde.id, 'id.p' = pf(m1.D.tilde.id, 1, m1.b.rho.df.id, 
  lower.tail = FALSE), 'log.rho' = m1.rho.bar.log, 
  'log.D.tilde' = m1.D.tilde.log, 'log.p' = pf(m1.D.tilde.log, 1, 
  m1.b.rho.df.log, lower.tail = FALSE), 'km.rho' = m1.rho.bar.km, 
  'km.D.tilde' = m1.D.tilde.km, 'km.p' = pf(m1.D.tilde.km, 1, m1.b.rho.df.km, 
  lower.tail = FALSE), 'rank.rho' = m1.rho.bar.rank, 
  'rank.D.tilde' = m1.D.tilde.rank, 'rank.p' = pf(m1.D.tilde.rank, 1, 
  m1.b.rho.df.rank, lower.tail = FALSE)), 3)

# model 1 -- coefficients

#m1.b <- c(m1.coef.L1, m1.coef.L2, m1.coef.L3, m1.coef.L4, m1.coef.L5)
#m1.s <- c(m1.se.L1, m1.se.L2, m1.se.L3, m1.se.L4, m1.se.L5)

#model1.imp <- mi.inference(m1.b, m1.s)

M <- 5

m2.rho.id <- cbind(fit.model2.L1.id$table[, 1], fit.model2.L2.id$table[, 1],
  fit.model2.L3.id$table[, 1], fit.model2.L4.id$table[, 1], 
  fit.model2.L5.id$table[, 1])
m2.rho.log <- cbind(fit.model2.L1.log$table[, 1], fit.model2.L2.log$table[, 1],
  fit.model2.L3.log$table[, 1], fit.model2.L4.log$table[, 1], 
  fit.model2.L5.log$table[, 1])
m2.rho.km <- cbind(fit.model2.L1.km$table[, 1], fit.model2.L2.km$table[, 1],
  fit.model2.L3.km$table[, 1], fit.model2.L4.km$table[, 1], 
  fit.model2.L5.km$table[, 1])
m2.rho.rank <- cbind(fit.model2.L1.rank$table[, 1], 
  fit.model2.L2.rank$table[, 1], fit.model2.L3.rank$table[, 1], 
  fit.model2.L4.rank$table[, 1], fit.model2.L5.rank$table[, 1])

m2.rho.chi.id <- cbind(fit.model2.L1.id$table[, 2], fit.model2.L2.id$table[, 2],
  fit.model2.L3.id$table[, 2], fit.model2.L4.id$table[, 2], 
  fit.model2.L5.id$table[, 2])
m2.rho.chi.log <- cbind(fit.model2.L1.log$table[, 2], 
  fit.model2.L2.log$table[, 2], fit.model2.L3.log$table[, 2], 
  fit.model2.L4.log$table[, 2], fit.model2.L5.log$table[, 2])
m2.rho.chi.km <- cbind(fit.model2.L1.km$table[, 2], fit.model2.L2.km$table[, 2],
  fit.model2.L3.km$table[, 2], fit.model2.L4.km$table[, 2], 
  fit.model2.L5.km$table[, 2])
m2.rho.chi.rank <- cbind(fit.model2.L1.rank$table[, 2], 
  fit.model2.L2.rank$table[, 2], fit.model2.L3.rank$table[, 2], 
  fit.model2.L4.rank$table[, 2], fit.model2.L5.rank$table[, 2])

m2.rho.bar.id <- apply(m2.rho.id, 1, mean) 
m2.rho.bar.log <- apply(m2.rho.log, 1, mean) 
m2.rho.bar.km <- apply(m2.rho.km, 1, mean) 
m2.rho.bar.rank <- apply(m2.rho.rank, 1, mean) 

m2.d.bar.id <- apply(m2.rho.chi.id, 1, mean)
m2.d.bar.log <- apply(m2.rho.chi.log, 1, mean)
m2.d.bar.km <- apply(m2.rho.chi.km, 1, mean)
m2.d.bar.rank <- apply(m2.rho.chi.rank, 1, mean)

m2.s2.rho.id <- apply(sqrt(m2.rho.chi.id), 1, var)
m2.s2.rho.log <- apply(sqrt(m2.rho.chi.log), 1, var)
m2.s2.rho.km <- apply(sqrt(m2.rho.chi.km), 1, var)
m2.s2.rho.rank <- apply(sqrt(m2.rho.chi.rank), 1, var)

m2.D.tilde.id <- 
  (m2.d.bar.id - (1 - 1 / M) * m2.s2.rho.id) / (1 + (1 + 1 / M) * m2.s2.rho.id)
m2.D.tilde.log <- 
  (m2.d.bar.log - (1 - 1 / M) * m2.s2.rho.log) / 
  (1 + (1 + 1 / M) * m2.s2.rho.log)
m2.D.tilde.km <- 
  (m2.d.bar.km - (1 - 1 / M) * m2.s2.rho.km) / (1 + (1 + 1 / M) * m2.s2.rho.km)
m2.D.tilde.rank <- 
  (m2.d.bar.rank - (1 - 1 / M) * m2.s2.rho.rank) / 
  (1 + (1 + 1 / M) * m2.s2.rho.rank)

m2.b.rho.df.id <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m2.s2.rho.id)^(-1))^2
m2.b.rho.df.log <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m2.s2.rho.log)^(-1))^2
m2.b.rho.df.km <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m2.s2.rho.km)^(-1))^2
m2.b.rho.df.rank <- 1^(-3 / M) * (M - 1) * 
  (1 + ((1 + 1 / M) * m2.s2.rho.rank)^(-1))^2

chiozzaandgoemans2004.ph.tab2 <- round(cbind('id.rho' = m2.rho.bar.id, 
  'id.D.tilde' = m2.D.tilde.id, 'id.p' = pf(m2.D.tilde.id, 1, m2.b.rho.df.id, 
  lower.tail = FALSE), 'log.rho' = m2.rho.bar.log, 
  'log.D.tilde' = m2.D.tilde.log, 'log.p' = pf(m2.D.tilde.log, 1, 
  m2.b.rho.df.log, lower.tail = FALSE), 'km.rho' = m2.rho.bar.km, 
  'km.D.tilde' = m2.D.tilde.km, 'km.p' = pf(m2.D.tilde.km, 1, m2.b.rho.df.km, 
  lower.tail = FALSE), 'rank.rho' = m2.rho.bar.rank, 
  'rank.D.tilde' = m2.D.tilde.rank, 'rank.p' = pf(m2.D.tilde.rank, 1, 
  m2.b.rho.df.rank, lower.tail = FALSE)), 3)

chiozzaandgoemans2004.ph.tab1
chiozzaandgoemans2004.ph.tab2
chiozzaandgoemans2004.ph.tab1[, c(3, 6, 9, 12)]
chiozzaandgoemans2004.ph.tab2[, c(3, 6, 9, 12)]

chiozzaandgoemans2004.var.labs.pretty1 <- c('Mixed Regime',
  'Parliamentary Democracy', 'Presidential Democracy', 'Civil War',
  'Economic Development', 'Chng. in Econ. Devel.', 'Trade Openness',
  'Change in Trade Openness', 'Population', 'Age', 'Previous Times in Office',
  'Crisis Involve. as Chall.', 'Crisis Involve. as Target',
  'Win Crisis', 'Lose Crisis', 'Draw Crisis', 'War Involve. as Chall.',
  'War Involve. as Target', 'Win War', 'Lose War', 'Draw War')

chiozzaandgoemans2004.var.labs.pretty2 <- c('Mixed Regime', 
  'Parliamentary Democracy', 'Presidential Democracy', 'Civil War',
  'Economic Development', 'Chng. in Econ. Devel.', 'Trade Openness',
  'Change in Trade Openness', 'Population', 'Age', 'Previous Times in Office',
  'Crisis Involve. as Chall.', 'Crisis Involve. as Target',
  'Autocracy$\\times$Win Crisis', 'Autocracy$\\times$Lose Crisis',
  'Autocracy$\\times$Draw Crisis', 'Mixed Reg.$\\times$Win Crisis',
  'Mixed Reg.$\\times$Lose Crisis', 'Mixed Reg.$\\times$Draw Crisis',
  'Parl. Democ.$\\times$Win Crisis', 'Parl. Democ.$\\times$Lose Crisis',
  'Parl. Democ.$\\times$Draw Crisis', 'Pres. Democ.$\\times$Win Crisis',
  'Pres. Democ.$\\times$Lose Crisis', 'Pres. Democ.$\\times$Draw Crisis',
  'War Involve. as Chall.', 'War Involve. as Target',
  'Autocracy$\\times$Win War', 'Autocracy$\\times$Lose War',
  'Autocracy$\\times$Draw War', 'Mixed Reg.$\\times$Win War',
  'Mixed Reg.$\\times$Lose War', 'Mixed Reg.$\\times$Draw War',
  'Parl. Democ.$\\times$Win War', 'Parl. Democ.$\\times$Lose War',
  'Parl. Democ.$\\times$Draw War', 'Pres. Democ.$\\times$Win War',
  'Pres. Democ.$\\times$Lose War', 'Pres. Democ.$\\times$Draw War')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(chiozzaandgoemans2004.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(chiozzaandgoemans2004.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( chiozzaandgoemans2004.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
chiozzaandgoemans2004.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(chiozzaandgoemans2004.ph.tab1)), ncol = 4)
rownames(chiozzaandgoemans2004.ph.tab.pretty1) <-
  c(chiozzaandgoemans2004.var.labs.pretty1, 'GLOBAL')
colnames(chiozzaandgoemans2004.ph.tab.pretty1) <- cols
for ( i in 1:nrow(chiozzaandgoemans2004.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    chiozzaandgoemans2004.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", chiozzaandgoemans2004.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  chiozzaandgoemans2004.ph.tab.pretty1[nrow(chiozzaandgoemans2004.ph.tab1), j] <-
    paste(sprintf("%.3f", chiozzaandgoemans2004.ph.tab1[nrow(chiozzaandgoemans2004.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(chiozzaandgoemans2004.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g06.txt')
  cat('Table SI.G.6', '\n\n')
  as.data.frame(chiozzaandgoemans2004.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.6 supplemental information', '\n\n')
  chiozzaandgoemans2004.ph.tab1
sink()

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(chiozzaandgoemans2004.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(chiozzaandgoemans2004.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( chiozzaandgoemans2004.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
chiozzaandgoemans2004.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(chiozzaandgoemans2004.ph.tab2)), ncol = 4)
rownames(chiozzaandgoemans2004.ph.tab.pretty2) <-
  c(chiozzaandgoemans2004.var.labs.pretty2, 'GLOBAL')
colnames(chiozzaandgoemans2004.ph.tab.pretty2) <- cols
for ( i in 1:nrow(chiozzaandgoemans2004.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    chiozzaandgoemans2004.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", chiozzaandgoemans2004.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  chiozzaandgoemans2004.ph.tab.pretty2[
    nrow(chiozzaandgoemans2004.ph.tab2), j] <-
      paste(sprintf("%.3f", chiozzaandgoemans2004.ph.tab2[
        nrow(chiozzaandgoemans2004.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(chiozzaandgoemans2004.ph.tab2), j],
      sep = '')
}

sink(file = 'tables/table-si-g07.txt')
  cat('Table SI.G.7', '\n\n')
  as.data.frame(chiozzaandgoemans2004.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.7 supplemental information', '\n\n')
  chiozzaandgoemans2004.ph.tab2
sink()
################################################################################


################################################################################
## 
## Crescenzi, Mark J. C. 2007. "Reputation and Interstate Conflict." American
##   Journal of Political Science 51(2): 382-396.
##
##     - results are identical to original paper
##

crescenzi2007 <- read.dta('data/nonpublic/crescenzi2007/ajps_rr_2.dta')

surv <- Surv(crescenzi2007$'_t0', crescenzi2007$'_t', crescenzi2007$'_d')
crescenzi2007.cox1 <- coxph(surv ~ I + contigdum + lcaprat + minmin + smldmat +
  S + cluster(dyadid), data = crescenzi2007, method = 'breslow')
crescenzi2007.cox2 <- coxph(surv ~ RISc_min + cluster(dyadid), 
  data = crescenzi2007, method = 'breslow')
crescenzi2007.cox3 <- coxph(surv ~ RISc_min + I + RI + contigdum + lcaprat +
  minmin + smldmat + S + cluster(dyadid), data = crescenzi2007, 
  method = 'breslow')
summary(crescenzi2007.cox1)
summary(crescenzi2007.cox2)
summary(crescenzi2007.cox3)

crescenzi2007.ph.tab1 <- zphTable(crescenzi2007.cox1, digits = 3)
crescenzi2007.ph.tab2 <- zphTable(crescenzi2007.cox2, digits = 3)
crescenzi2007.ph.tab3 <- zphTable(crescenzi2007.cox3, digits = 3)
crescenzi2007.ph.tab1
crescenzi2007.ph.tab1[, c(3, 6, 9, 12)]
crescenzi2007.ph.tab2
crescenzi2007.ph.tab2[, c(3, 6, 9, 12)]
crescenzi2007.ph.tab3
crescenzi2007.ph.tab3[, c(3, 6, 9, 12)]

crescenzi2007.var.labs1 <- rownames(crescenzi2007.ph.tab1)
crescenzi2007.var.labs2 <- rownames(crescenzi2007.ph.tab2)
crescenzi2007.var.labs3 <- rownames(crescenzi2007.ph.tab3)
crescenzi2007.var.labs.pretty1 <- c('IIS', 'Contiguous', 
  'Minor Powers', 'log(Capability Ratio)', 'Regime Score', 'S-Score')
crescenzi2007.var.labs.pretty2 <- c('RISc')
crescenzi2007.var.labs.pretty3 <- c('RISc', 'IIS', 'RISc*IIS', 'Contiguous',
  'Minor Powers', 'log(Capability Ratio)', 'Regime Score', 'S-Score')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(crescenzi2007.ph.tab1)), ncol = 4)

for ( i in 1:nrow(crescenzi2007.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( crescenzi2007.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
crescenzi2007.ph.tab.pretty1 <- matrix(rep(NA, 4 * nrow(crescenzi2007.ph.tab1)),
  ncol = 4)
rownames(crescenzi2007.ph.tab.pretty1) <- c(crescenzi2007.var.labs.pretty1,
  'GLOBAL')
colnames(crescenzi2007.ph.tab.pretty1) <- cols
for ( i in 1:nrow(crescenzi2007.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    crescenzi2007.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", crescenzi2007.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  crescenzi2007.ph.tab.pretty1[nrow(crescenzi2007.ph.tab1), j] <-
    paste(sprintf("%.3f", crescenzi2007.ph.tab1[nrow(crescenzi2007.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(crescenzi2007.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g08.txt')
  cat('Table SI.G.8', '\n\n')
  as.data.frame(crescenzi2007.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.8 supplemental information', '\n\n')
  crescenzi2007.ph.tab1
sink()

zphPlot1(cox.fit = crescenzi2007.cox1, var.lab = 'lcaprat',
  var.lab.pretty = 'log(Capability Ratio)', directory = 'figures',
  fig.name = 'figure-si-g08')

zphPlots(cox.fit = crescenzi2007.cox1, var.labs = crescenzi2007.var.labs1,
  var.labs.pretty = crescenzi2007.var.labs.pretty1,
  directory = 'figures/supplemental/crescenzi2007/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(crescenzi2007.ph.tab2)), ncol = 4)

for ( i in 1:nrow(crescenzi2007.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( crescenzi2007.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
crescenzi2007.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * (nrow(crescenzi2007.ph.tab2) + 1)), ncol = 4)
rownames(crescenzi2007.ph.tab.pretty2) <- c(crescenzi2007.var.labs.pretty2,
  'GLOBAL')
colnames(crescenzi2007.ph.tab.pretty2) <- cols
for ( i in 1:nrow(crescenzi2007.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    crescenzi2007.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", crescenzi2007.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  crescenzi2007.ph.tab.pretty2[nrow(crescenzi2007.ph.tab2), j] <-
    paste(sprintf("%.3f", crescenzi2007.ph.tab2[nrow(crescenzi2007.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(crescenzi2007.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g09.txt')
  cat('Table SI.G.9', '\n\n')
  as.data.frame(crescenzi2007.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.9 supplemental information', '\n\n')
  crescenzi2007.ph.tab2
sink()

zphPlots(cox.fit = crescenzi2007.cox2, var.labs = crescenzi2007.var.labs2,
  var.labs.pretty = crescenzi2007.var.labs.pretty2,
  directory = 'figures/supplemental/crescenzi2007/model2')

star.tab.temp3 <- matrix(rep(NA, 4 * nrow(crescenzi2007.ph.tab3)), ncol = 4)

for ( i in 1:nrow(crescenzi2007.ph.tab3) ) {
  for ( j in 1:4 ) {
    ifelse ( crescenzi2007.ph.tab3[i, p.val.cols[j]] < .05,
      star.tab.temp3[i, j] <- '*', star.tab.temp3[i, j] <- ' ' )
  }
}
crescenzi2007.ph.tab.pretty3 <- matrix(rep(NA, 4 * nrow(crescenzi2007.ph.tab3)),
  ncol = 4)
rownames(crescenzi2007.ph.tab.pretty3) <- c(crescenzi2007.var.labs.pretty3,
  'GLOBAL')
colnames(crescenzi2007.ph.tab.pretty3) <- cols
for ( i in 1:nrow(crescenzi2007.ph.tab3) - 1 ) {
  for ( j in 1:4 ) {
    crescenzi2007.ph.tab.pretty3[i, j] <-
      paste(sprintf("%.3f", crescenzi2007.ph.tab3[i, rho.cols[j]]),
      star.tab.temp3[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  crescenzi2007.ph.tab.pretty3[nrow(crescenzi2007.ph.tab3), j] <-
    paste(sprintf("%.3f", crescenzi2007.ph.tab3[nrow(crescenzi2007.ph.tab3),
    chi.sq.cols[j]]), star.tab.temp3[nrow(crescenzi2007.ph.tab3), j], sep = '')
}

sink(file = 'tables/table-si-g10.txt')
  cat('Table SI.G.10', '\n\n')
  as.data.frame(crescenzi2007.ph.tab.pretty3)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.10 supplemental information', '\n\n')
  crescenzi2007.ph.tab3
sink()

zphPlot1(cox.fit = crescenzi2007.cox3, var.lab = 'RI',
  var.lab.pretty = 'RISc*IIS', directory = 'figures',
  fig.name = 'figure-si-g09')
zphPlot1(cox.fit = crescenzi2007.cox3, var.lab = 'lcaprat',
  var.lab.pretty = 'log(Capability Ratio)', directory = 'figures',
  fig.name = 'figure-si-g10')

zphPlots(cox.fit = crescenzi2007.cox3, var.labs = crescenzi2007.var.labs3,
  var.labs.pretty = crescenzi2007.var.labs.pretty3,
  directory = 'figures/supplemental/crescenzi2007/model3')
################################################################################


################################################################################
##
## Cunningham, Kathleen Gallagher. 2011. "Divide and Conquer or Divide and
##   Concede: How Do States Respond to Internally Divided Separatists?" American
##   Political Science Review 105(2): 275-297.
##
##     - results identical to original paper
##

cunningham2011 <-
  read.dta('data/nonpublic/cunningham2011/APSR duration analysis final.dta')

surv1 <- with(cunningham2011, Surv(duration, violence))
cunningham2011.cox1 <- coxph(surv1 ~ unitary2 + newcombosize + base + logpop +
  loggdppc + milexpc + strata(numberconcession) + cluster(caseid), 
  method = 'breslow', data = cunningham2011)
cunningham2011.cox2 <- coxph(surv1 ~ logfactions + newcombosize + base + 
  logpop + loggdppc + milexpc + strata(numberconcession) + cluster(caseid), 
  method = 'breslow', data = cunningham2011)

surv2 <- with(cunningham2011, Surv(duration, concessionend_l))
cunningham2011.cox3 <- coxph(surv2 ~ unitary2 + newcombosize + base + logpop +
  loggdppc + milexpc + strata(numberconcession) + cluster(caseid), 
  ties = 'breslow', data = cunningham2011)
cunningham2011.cox4 <- coxph(surv2 ~ logfactions + newcombosize + base + 
  logpop + loggdppc + milexpc + strata(numberconcession) + cluster(caseid), 
  ties = 'breslow', data = cunningham2011)

summary(cunningham2011.cox1)
summary(cunningham2011.cox2)
summary(cunningham2011.cox3)
summary(cunningham2011.cox4)

cunningham2011.ph.tab1 <- zphTable(cunningham2011.cox1, digits = 3)
cunningham2011.ph.tab2 <- zphTable(cunningham2011.cox2, digits = 3)
cunningham2011.ph.tab3 <- zphTable(cunningham2011.cox3, digits = 3)
cunningham2011.ph.tab4 <- zphTable(cunningham2011.cox4, digits = 3)
cunningham2011.ph.tab1
cunningham2011.ph.tab1[, c(3, 6, 9, 12)]
cunningham2011.ph.tab2
cunningham2011.ph.tab2[, c(3, 6, 9, 12)]
cunningham2011.ph.tab3
cunningham2011.ph.tab3[, c(3, 6, 9, 12)]
cunningham2011.ph.tab4
cunningham2011.ph.tab4[, c(3, 6, 9, 12)]

cunningham2011.var.labs1 <- rownames(cunningham2011.ph.tab1)
cunningham2011.var.labs2 <- rownames(cunningham2011.ph.tab3)
cunningham2011.var.labs.pretty1 <- c('Unitary Movement', 
  'Relative Size of Group', 'Territorial Base', 'log(State Population)',
  'log(GDP Per Capita)', 'Military Expenditure Per Capita')
cunningham2011.var.labs.pretty2 <- c('log(Number of SD Factions)', 
  'Relative Size of Group', 'Territorial Base', 'log(State Population)',
  'log(GDP Per Capita)', 'Military Expenditure Per Capita')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab1)), ncol = 4)

for ( i in 1:nrow(cunningham2011.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( cunningham2011.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
cunningham2011.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab1)), ncol = 4)
rownames(cunningham2011.ph.tab.pretty1) <- c(cunningham2011.var.labs.pretty1,
  'GLOBAL')
colnames(cunningham2011.ph.tab.pretty1) <- cols
for ( i in 1:nrow(cunningham2011.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    cunningham2011.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", cunningham2011.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  cunningham2011.ph.tab.pretty1[nrow(cunningham2011.ph.tab1), j] <-
    paste(sprintf("%.3f", cunningham2011.ph.tab1[nrow(cunningham2011.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(cunningham2011.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g11.txt')
  cat('Table SI.G.11', '\n\n')
  as.data.frame(cunningham2011.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.11 supplemental information', '\n\n')
  cunningham2011.ph.tab1
sink()

zphPlot1(cox.fit = cunningham2011.cox1, var.lab = 'unitary2',
  var.lab.pretty = 'Unitary Movement', directory = 'figures',
  fig.name = 'figure-si-g11')

zphPlots(cox.fit = cunningham2011.cox1, var.labs = cunningham2011.var.labs1,
  var.labs.pretty = cunningham2011.var.labs.pretty1,
  directory = 'figures/supplemental/cunningham2011/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab2)), ncol = 4)

for ( i in 1:nrow(cunningham2011.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( cunningham2011.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
cunningham2011.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab2)), ncol = 4)
rownames(cunningham2011.ph.tab.pretty2) <- c(cunningham2011.var.labs.pretty2,
  'GLOBAL')
colnames(cunningham2011.ph.tab.pretty2) <- cols
for ( i in 1:nrow(cunningham2011.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    cunningham2011.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", cunningham2011.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  cunningham2011.ph.tab.pretty2[nrow(cunningham2011.ph.tab2), j] <-
    paste(sprintf("%.3f", cunningham2011.ph.tab2[nrow(cunningham2011.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(cunningham2011.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g12.txt')
  cat('Table SI.G.12', '\n\n')
  as.data.frame(cunningham2011.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.12 supplemental information', '\n\n')
  cunningham2011.ph.tab2
sink()

zphPlots(cox.fit = cunningham2011.cox2, var.labs = cunningham2011.var.labs2,
  var.labs.pretty = cunningham2011.var.labs.pretty2,
  directory = 'figures/supplemental/cunningham2011/model2')

star.tab.temp3 <- matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab3)), ncol = 4)

for ( i in 1:nrow(cunningham2011.ph.tab3) ) {
  for ( j in 1:4 ) {
    ifelse ( cunningham2011.ph.tab3[i, p.val.cols[j]] < .05,
      star.tab.temp3[i, j] <- '*', star.tab.temp3[i, j] <- ' ' )
  }
}
cunningham2011.ph.tab.pretty3 <-
  matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab3)), ncol = 4)
rownames(cunningham2011.ph.tab.pretty3) <- c(cunningham2011.var.labs.pretty1,
  'GLOBAL')
colnames(cunningham2011.ph.tab.pretty3) <- cols
for ( i in 1:nrow(cunningham2011.ph.tab3) - 1 ) {
  for ( j in 1:4 ) {
    cunningham2011.ph.tab.pretty3[i, j] <-
      paste(sprintf("%.3f", cunningham2011.ph.tab3[i, rho.cols[j]]),
      star.tab.temp3[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  cunningham2011.ph.tab.pretty3[nrow(cunningham2011.ph.tab3), j] <-
    paste(sprintf("%.3f", cunningham2011.ph.tab3[nrow(cunningham2011.ph.tab3),
    chi.sq.cols[j]]), star.tab.temp3[nrow(cunningham2011.ph.tab3), j], sep = '')
}

sink(file = 'tables/table-si-g13.txt')
  cat('Table SI.G.13', '\n\n')
  as.data.frame(cunningham2011.ph.tab.pretty3)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.13 supplemental information', '\n\n')
  cunningham2011.ph.tab3
sink()

zphPlot1(cox.fit = cunningham2011.cox3, var.lab = 'loggdppc',
  var.lab.pretty = 'log(GDP Per Capita)', directory = 'figures',
  fig.name = 'figure-si-g12')

zphPlots(cox.fit = cunningham2011.cox3, var.labs = cunningham2011.var.labs1,
  var.labs.pretty = cunningham2011.var.labs.pretty1,
  directory = 'figures/supplemental/cunningham2011/model3')

star.tab.temp4 <- matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab4)), ncol = 4)

for ( i in 1:nrow(cunningham2011.ph.tab4) ) {
  for ( j in 1:4 ) {
    ifelse ( cunningham2011.ph.tab4[i, p.val.cols[j]] < .05,
      star.tab.temp4[i, j] <- '*', star.tab.temp4[i, j] <- ' ' )
  }
}
cunningham2011.ph.tab.pretty4 <-
  matrix(rep(NA, 4 * nrow(cunningham2011.ph.tab4)), ncol = 4)
rownames(cunningham2011.ph.tab.pretty4) <- c(cunningham2011.var.labs.pretty2,
  'GLOBAL')
colnames(cunningham2011.ph.tab.pretty4) <- cols
for ( i in 1:nrow(cunningham2011.ph.tab4) - 1 ) {
  for ( j in 1:4 ) {
    cunningham2011.ph.tab.pretty4[i, j] <-
      paste(sprintf("%.3f", cunningham2011.ph.tab4[i, rho.cols[j]]),
      star.tab.temp4[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  cunningham2011.ph.tab.pretty4[nrow(cunningham2011.ph.tab4), j] <-
    paste(sprintf("%.3f", cunningham2011.ph.tab4[nrow(cunningham2011.ph.tab4),
    chi.sq.cols[j]]), star.tab.temp4[nrow(cunningham2011.ph.tab4), j], sep = '')
}

sink(file = 'tables/table-si-g14.txt')
  cat('Table SI.G.14', '\n\n')
  as.data.frame(cunningham2011.ph.tab.pretty4)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.14 supplemental information', '\n\n')
  cunningham2011.ph.tab4
sink()

zphPlot1(cox.fit = cunningham2011.cox4, var.lab = 'newcombosize',
  var.lab.pretty = 'Relative Size of Group', directory = 'figures',
  fig.name = 'figure-si-g13')
zphPlot1(cox.fit = cunningham2011.cox4, var.lab = 'loggdppc',
  var.lab.pretty = 'log(GDP Per Capita)', directory = 'figures',
  fig.name = 'figure-si-g13')

zphPlots(cox.fit = cunningham2011.cox4, var.labs = cunningham2011.var.labs2,
  var.labs.pretty = cunningham2011.var.labs.pretty2,
  directory = 'figures/supplemental/cunningham2011/model4')
################################################################################


################################################################################
##
## Debs, Alexandre, and H.E. Goemans. 2010. "Regime Type, the Fate of Leaders,
##   and War." American Political Science Review 104(3): 430-445.
##
##     - results identical to original paper
##     - only pre-presentation models estimated
##     - identified possible mistake in interpretation of ph tests from article 
##       (civil war should not be interacted w/ time)
##

debsandgoemans2010.1 <-
  read.dta('data/nonpublic/debsandgoemans2010/Debs2009.dta',
  convert.dates = FALSE)
debsandgoemans2010.2 <-
  read.dta('data/nonpublic/debsandgoemans2010/PGtoR.dta',
  convert.dates = FALSE)

#this is required for numerical stability; amount of listwise deletion when 
#  using raw data causes cox.zph function to fail
debsandgoemans2010.1 <- drop(subset(debsandgoemans2010.1, !is.na(t0) & 
  !is.na(t) & !is.na(d) & !is.na(democracy) & !is.na(civwar) & 
  !is.na(lngdpcap) & !is.na(growth) & !is.na(tropen4) & !is.na(dopen4) & 
  !is.na(lnpop) & !is.na(age0) & !is.na(powtimes) & !is.na(entry1) & 
  !is.na(initiator2) & !is.na(defender2) & !is.na(inherit) & !is.na(demdwinsh) & 
  !is.na(demdlosesh) & !is.na(demddrawsh) & !is.na(nondemdwinsh) & 
  !is.na(nondemdlosesh) & !is.na(nondemddrawsh) & !is.na(demdwinwar) & 
  !is.na(demdlosewar) & !is.na(demddrawwar) & !is.na(nondemdwinwar) & 
  !is.na(nondemdlosewar) & !is.na(nondemddrawwar)))
  
surv1 <- with(debsandgoemans2010.1, Surv(t0, t, d))

debsandgoemans2010.cox1 <- coxph(surv1 ~ democracy + civwar + lngdpcap + 
  growth + tropen4 + dopen4 + lnpop + age0 + powtimes + entry1 + initiator2 + 
  defender2 + inherit + demdwinsh + demdlosesh + demddrawsh + nondemdwinsh + 
  nondemdlosesh + nondemddrawsh + demdwinwar + demdlosewar + demddrawwar + 
  nondemdwinwar + nondemdlosewar + nondemddrawwar + frailty(ccode), 
  na.action = na.exclude, data = debsandgoemans2010.1, x = TRUE,
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#debsandgoemans2010.1$txdemocracy <- 
#  with(debsandgoemans2010.1, log(t) * democracy)
#debsandgoemans2010.1$txcivwar <- with(debsandgoemans2010.1, log(t) * civwar)
#debsandgoemans2010.1$txtropen4 <- with(debsandgoemans2010.1, log(t) * tropen4)
#debsandgoemans2010.1$txage0 <- with(debsandgoemans2010.1, log(t) * age0)
#debsandgoemans2010.1$txentry1 <- with(debsandgoemans2010.1, log(t) * entry1)
#debsandgoemans2010.1$txnondemdlosesh <- 
#  with(debsandgoemans2010.1, log(t) * nondemdlosesh)
#debsandgoemans2010.1$txnondemddrawsh <- 
#  with(debsandgoemans2010.1, log(t) * nondemddrawsh)

#debsandgoemans2010.cox1.tx <- coxph(surv1 ~ democracy + txdemocracy + civwar +
#  txcivwar + lngdpcap + growth + tropen4 + dopen4 + txtropen4 + lnpop + age0 + 
#  txage0 + powtimes + entry1 + txentry1 + initiator2 + defender2 + inherit + 
#  demdwinsh + demdlosesh + demddrawsh + nondemdwinsh + nondemdlosesh + 
#  txnondemdlosesh + nondemddrawsh + txnondemddrawsh + demdwinwar + demdlosewar + 
#  demddrawwar + nondemdwinwar + nondemdlosewar + nondemddrawwar + 
#  frailty(ccode), na.action = na.exclude, data = debsandgoemans2010.1, 
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))
#summary(debsandgoemans2010.cox1.tx)

#this is required for numerical stability; amount of listwise deletion when 
#  using raw data causes cox.zph function to fail
debsandgoemans2010.2 <- drop(subset(debsandgoemans2010.2, !is.na(t0) &
  !is.na(t) & !is.na(d) & !is.na(civilian) & !is.na(monarch) & !is.na(civwar) &
  !is.na(lngdpcap) & !is.na(growth) & !is.na(tropen4) & !is.na(dopen4) &
  !is.na(lnpop) & !is.na(age0) & !is.na(entry1) & !is.na(powtimes) &
  !is.na(initiator2) & !is.na(defender2) & !is.na(inherit) & !is.na(milwinsh) &
  !is.na(millosesh) & !is.na(mildrawsh) & !is.na(milwinwar) &
  !is.na(millosewar) & !is.na(mildrawwar) & !is.na(civwinsh) &
  !is.na(civlosesh) & !is.na(civdrawsh) & !is.na(civwinwar) &
  !is.na(civlosewar) & !is.na(civdrawwar) & !is.na(monwinsh) &
  !is.na(monlosesh) & !is.na(mondrawsh) & !is.na(monwinwar) &
  !is.na(monlosewar) & !is.na(mondrawwar)))

surv2 <- with(debsandgoemans2010.2, Surv(t0, t, d))

debsandgoemans2010.cox2 <- coxph(surv2 ~ civilian + monarch + civwar + 
  lngdpcap + growth + tropen4 + dopen4 + lnpop + age0 + entry1 + powtimes + 
  initiator2 + defender2 + inherit + milwinsh + millosesh + mildrawsh + 
  milwinwar + millosewar + mildrawwar + civwinsh + civlosesh + civdrawsh + 
  civwinwar + civlosewar + civdrawwar + monwinsh + monlosesh + mondrawsh + 
  monwinwar + monlosewar + mondrawwar + frailty(ccode), na.action = na.exclude, 
  data = debsandgoemans2010.2,
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

#debsandgoemans2010.2$txlngdpcap <-
#  with(debsandgoemans2010.2, log(t) * lngdpcap)
#debsandgoemans2010.2$txentry1 <- with(debsandgoemans2010.2, log(t) * entry1)
#
#debsandgoemans2010.cox2.tx <- coxph(surv2 ~ civilian + monarch + civwar + 
#  lngdpcap + txlngdpcap + growth + tropen4 + dopen4 + lnpop + age0 + entry1 + 
#  txentry1 + powtimes + initiator2 + defender2 + inherit + milwinsh + 
#  millosesh + mildrawsh + milwinwar + millosewar + mildrawwar + civwinsh + 
#  civlosesh + civdrawsh + civwinwar + civlosewar + civdrawwar + monwinsh + 
#  monlosesh + mondrawsh + monwinwar + monlosewar + mondrawwar + frailty(ccode), 
#  na.action = na.exclude, data = debsandgoemans2010.2,
#  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))
#summary(debsandgoemans2010.cox2.tx)

summary(debsandgoemans2010.cox1)
summary(debsandgoemans2010.cox2)

debsandgoemans2010.ph.tab1 <- zphTable(debsandgoemans2010.cox1, digits = 3)
debsandgoemans2010.ph.tab2 <- zphTable(debsandgoemans2010.cox2, digits = 3)
debsandgoemans2010.ph.tab1
debsandgoemans2010.ph.tab1[, c(3, 6, 9, 12)]
debsandgoemans2010.ph.tab2
debsandgoemans2010.ph.tab2[, c(3, 6, 9, 12)]

debsandgoemans2010.var.labs1 <- rownames(debsandgoemans2010.ph.tab1)
debsandgoemans2010.var.labs.pretty1 <- c('Democracy', 'Civil War', 
  'GDP Per Capita', 'GDP Growth', 'Trade Openness', 'Chng. in Trade Open.',
  'log(Population)', 'Age', 'Times in Office', 'Entry', 'Challenger', 'Target',
  'Inheritor', 'Democ. Crisis Victory', 'Democ. Crisis Defeat', 
  'Democ. Crisis Draw', 'Nondemoc. Crisis Victory', 
  'Nondemoc. Crisis Defeat', 'Nondemoc. Crisis Draw', 
  'Democ. War Victory', 'Democ. War Defeat', 'Democ. War Draw', 
  'Nondemoc. War Victory', 'Nondemoc. War Defeat', 
  'Nondemoc. War Draw')

debsandgoemans2010.var.labs2 <- rownames(debsandgoemans2010.ph.tab2)
debsandgoemans2010.var.labs.pretty2 <- c('Civilian', 'Monarch', 'Civil War',
  'GDP Per Capita', 'Growth', 'Trade Openness', 'Chng. in Trade Open.',
  'log(Population)', 'Age at Entry', 'Manner of Entry', 'Times in Office',
  'Challenger', 'Target', 'Inheritor', 'Military Crisis Victory',
  'Military Crisis Defeat', 'Military Crisis Draw', 'Military War Victory',
  'Military War Defeat', 'Military War Draw', 'Civilian Crisis Victory',
  'Civilian Crisis Defeat', 'Civilian Crisis Draw', 'Civilian War Victory',
  'Civilian War Defeat', 'Civilian War Draw', 'Monarch Crisis Victory',
  'Monarch Crisis Defeat', 'Monarch Crisis Draw', 'Monarch War Victory',
  'Monarch War Defeat', 'Monarch War Draw')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(debsandgoemans2010.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(debsandgoemans2010.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( debsandgoemans2010.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
debsandgoemans2010.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(debsandgoemans2010.ph.tab1)), ncol = 4)
rownames(debsandgoemans2010.ph.tab.pretty1) <-
  c(debsandgoemans2010.var.labs.pretty1, 'GLOBAL')
colnames(debsandgoemans2010.ph.tab.pretty1) <- cols
for ( i in 1:nrow(debsandgoemans2010.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    debsandgoemans2010.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", debsandgoemans2010.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  debsandgoemans2010.ph.tab.pretty1[nrow(debsandgoemans2010.ph.tab1), j] <-
    paste(sprintf("%.3f", debsandgoemans2010.ph.tab1[
      nrow(debsandgoemans2010.ph.tab1), chi.sq.cols[j]]),
    star.tab.temp1[nrow(debsandgoemans2010.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g15.txt')
  cat('Table SI.G.15', '\n\n')
  as.data.frame(debsandgoemans2010.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.15 supplemental information', '\n\n')
  debsandgoemans2010.ph.tab1
sink()

zphPlot1(cox.fit = debsandgoemans2010.cox1, var.lab = 'civwar',
  var.lab.pretty = 'Civil War', directory = 'figures',
  fig.name = 'figure-si-g15')
zphPlot1(cox.fit = debsandgoemans2010.cox1, var.lab = 'lngdpcap',
  var.lab.pretty = 'GDP Per Capita', directory = 'figures',
  fig.name = 'figure-si-g16')
zphPlot1(cox.fit = debsandgoemans2010.cox1, var.lab = 'tropen4',
  var.lab.pretty = 'Trade Openness', directory = 'figures',
  fig.name = 'figure-si-g17')
zphPlot1(cox.fit = debsandgoemans2010.cox1, var.lab = 'nondemdlosesh',
  var.lab.pretty = 'Nondemoc. Crisis Defeat', directory = 'figures',
  fig.name = 'figure-si-g18')
zphPlot1(cox.fit = debsandgoemans2010.cox1, var.lab = 'nondemdlosewar',
  var.lab.pretty = 'Nondemoc. War Defeat', directory = 'figures',
  fig.name = 'figure-si-g19')

zphPlots(cox.fit = debsandgoemans2010.cox1, 
  var.labs = debsandgoemans2010.var.labs1,
  var.labs.pretty = debsandgoemans2010.var.labs.pretty1,
  directory = 'figures/supplemental/debsandgoemans2010/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(debsandgoemans2010.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(debsandgoemans2010.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( debsandgoemans2010.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
debsandgoemans2010.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(debsandgoemans2010.ph.tab2)), ncol = 4)
rownames(debsandgoemans2010.ph.tab.pretty2) <-
  c(debsandgoemans2010.var.labs.pretty2, 'GLOBAL')
colnames(debsandgoemans2010.ph.tab.pretty2) <- cols
for ( i in 1:nrow(debsandgoemans2010.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    debsandgoemans2010.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", debsandgoemans2010.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  debsandgoemans2010.ph.tab.pretty2[nrow(debsandgoemans2010.ph.tab2), j] <-
    paste(sprintf("%.3f", debsandgoemans2010.ph.tab2[
      nrow(debsandgoemans2010.ph.tab2), chi.sq.cols[j]]),
    star.tab.temp2[nrow(debsandgoemans2010.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g16.txt')
  cat('Table SI.G.16', '\n\n')
  as.data.frame(debsandgoemans2010.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.16 supplemental information', '\n\n')
  debsandgoemans2010.ph.tab2
sink()

zphPlot1(cox.fit = debsandgoemans2010.cox2, var.lab = 'lngdpcap',
  var.lab.pretty = 'GDP Per Capita', directory = 'figures',
  fig.name = 'figure-si-g20')
zphPlot1(cox.fit = debsandgoemans2010.cox2, var.lab = 'age0',
  var.lab.pretty = 'Age at Entry', directory = 'figures',
  fig.name = 'figure-si-g21')

zphPlots(cox.fit = debsandgoemans2010.cox2,
  var.labs = debsandgoemans2010.var.labs2,
  var.labs.pretty = debsandgoemans2010.var.labs.pretty2,
  directory = 'figures/supplemental/debsandgoemans2010/model2')
################################################################################


################################################################################
##
## Diermeier, Daniel, and Randy T. Stevenson. 1999. "Cabinet Survival and 
##   Competing Risks." American Journal of Political Science 43(4): 1051-1068.
##
##     - results identical to original paper (except opposite sign for majority 
##       status coefficient for all models; likely a mistake in the original 
##       article)
##

diermeierandstevenson1999 <- read.dta(
  'data/nonpublic/diermeierandstevenson1999/DiermeierStevensonAJPS1999RepData.dta')

surv1 <- with(diermeierandstevenson1999, Surv(dur2, cenwar))
diermeierandstevenson1999.cox1 <- coxph(surv1 ~ cmajg + postel + invest + crn2 +
  callrdr + cd2r + cd3r, data = diermeierandstevenson1999)  

surv2 <- with(diermeierandstevenson1999, Surv(dur2, elcenwar))
diermeierandstevenson1999.cox2 <- coxph(surv2 ~ cmajg + postel + invest + crn2 +
  callrdr+ cd2r + cd3r, data = diermeierandstevenson1999)

surv3 <- with(diermeierandstevenson1999, Surv(dur2, eleccens))
diermeierandstevenson1999.cox3 <- coxph(surv3 ~ cmajg + postel + invest + crn2 +
  callrdr + cd2r + cd3r, data = diermeierandstevenson1999)

surv4 <- with(diermeierandstevenson1999, Surv(dur2, bargcens))
diermeierandstevenson1999.cox4 <- coxph(surv4 ~ cmajg + postel + invest + crn2 +
  callrdr + cd2r + cd3r, data = diermeierandstevenson1999)

summary(diermeierandstevenson1999.cox1)
summary(diermeierandstevenson1999.cox2)
summary(diermeierandstevenson1999.cox3)
summary(diermeierandstevenson1999.cox4)

diermeierandstevenson1999.ph.tab1 <- zphTable(diermeierandstevenson1999.cox1, 
  digits = 3)
diermeierandstevenson1999.ph.tab2 <- zphTable(diermeierandstevenson1999.cox2,
  digits = 3)
diermeierandstevenson1999.ph.tab3 <- zphTable(diermeierandstevenson1999.cox3,
  digits = 3)
diermeierandstevenson1999.ph.tab4 <- zphTable(diermeierandstevenson1999.cox4,
  digits = 3)
diermeierandstevenson1999.ph.tab1
diermeierandstevenson1999.ph.tab1[, c(3, 6, 9, 12)]
diermeierandstevenson1999.ph.tab2
diermeierandstevenson1999.ph.tab2[, c(3, 6, 9, 12)]
diermeierandstevenson1999.ph.tab3
diermeierandstevenson1999.ph.tab3[, c(3, 6, 9, 12)]
diermeierandstevenson1999.ph.tab4
diermeierandstevenson1999.ph.tab4[, c(3, 6, 9, 12)]

diermeierandstevenson1999.var.labs <-
  rownames(diermeierandstevenson1999.ph.tab1)
diermeierandstevenson1999.var.labs.pretty <- c('Majority Status', 
  'Post-Election', 'Investiture Vote', 'Returnability', 'Left-Right Diversity',
  'Clerical-Secular Diversity', 'Regime Support Diversity')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(diermeierandstevenson1999.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( diermeierandstevenson1999.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
diermeierandstevenson1999.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab1)), ncol = 4)
rownames(diermeierandstevenson1999.ph.tab.pretty1) <-
  c(diermeierandstevenson1999.var.labs.pretty, 'GLOBAL')
colnames(diermeierandstevenson1999.ph.tab.pretty1) <- cols
for ( i in 1:nrow(diermeierandstevenson1999.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    diermeierandstevenson1999.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  diermeierandstevenson1999.ph.tab.pretty1[
    nrow(diermeierandstevenson1999.ph.tab1), j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab1[
        nrow(diermeierandstevenson1999.ph.tab1), chi.sq.cols[j]]),
        star.tab.temp1[nrow(diermeierandstevenson1999.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g17.txt')
  cat('Table SI.G.17', '\n\n')
  as.data.frame(diermeierandstevenson1999.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.17 supplemental information', '\n\n')
  diermeierandstevenson1999.ph.tab1
sink()

zphPlot1(cox.fit = diermeierandstevenson1999.cox1, var.lab = 'cmajg',
  var.lab.pretty = 'Majority Status', directory = 'figures',
  fig.name = 'figure-si-g22')
zphPlot1(cox.fit = diermeierandstevenson1999.cox1, var.lab = 'crn2',
  var.lab.pretty = 'Returnability', directory = 'figures',
  fig.name = 'figure-si-g23')

zphPlots(cox.fit = diermeierandstevenson1999.cox1,
  var.labs = diermeierandstevenson1999.var.labs,
  var.labs.pretty = diermeierandstevenson1999.var.labs.pretty,
  directory = 'figures/supplemental/diermeierandstevenson1999/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(diermeierandstevenson1999.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( diermeierandstevenson1999.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
diermeierandstevenson1999.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab2)), ncol = 4)
rownames(diermeierandstevenson1999.ph.tab.pretty2) <-
  c(diermeierandstevenson1999.var.labs.pretty, 'GLOBAL')
colnames(diermeierandstevenson1999.ph.tab.pretty2) <- cols
for ( i in 1:nrow(diermeierandstevenson1999.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    diermeierandstevenson1999.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  diermeierandstevenson1999.ph.tab.pretty2[
    nrow(diermeierandstevenson1999.ph.tab2), j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab2[
        nrow(diermeierandstevenson1999.ph.tab2), chi.sq.cols[j]]),
      star.tab.temp2[nrow(diermeierandstevenson1999.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g18.txt')
  cat('Table SI.G.18', '\n\n')
  as.data.frame(diermeierandstevenson1999.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.18 supplemental information', '\n\n')
  diermeierandstevenson1999.ph.tab2
sink()

zphPlots(cox.fit = diermeierandstevenson1999.cox2,
  var.labs = diermeierandstevenson1999.var.labs,
  var.labs.pretty = diermeierandstevenson1999.var.labs.pretty,
  directory = 'figures/supplemental/diermeierandstevenson1999/model2')

star.tab.temp3 <- matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab3)),
  ncol = 4)

for ( i in 1:nrow(diermeierandstevenson1999.ph.tab3) ) {
  for ( j in 1:4 ) {
    ifelse ( diermeierandstevenson1999.ph.tab3[i, p.val.cols[j]] < .05,
      star.tab.temp3[i, j] <- '*', star.tab.temp3[i, j] <- ' ' )
  }
}
diermeierandstevenson1999.ph.tab.pretty3 <-
  matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab3)), ncol = 4)
rownames(diermeierandstevenson1999.ph.tab.pretty3) <-
  c(diermeierandstevenson1999.var.labs.pretty, 'GLOBAL')
colnames(diermeierandstevenson1999.ph.tab.pretty3) <- cols
for ( i in 1:nrow(diermeierandstevenson1999.ph.tab3) - 1 ) {
  for ( j in 1:4 ) {
    diermeierandstevenson1999.ph.tab.pretty3[i, j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab3[i, rho.cols[j]]),
      star.tab.temp3[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  diermeierandstevenson1999.ph.tab.pretty3[
    nrow(diermeierandstevenson1999.ph.tab3), j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab3[
        nrow(diermeierandstevenson1999.ph.tab3), chi.sq.cols[j]]),
        star.tab.temp3[nrow(diermeierandstevenson1999.ph.tab3), j], sep = '')
}

sink(file = 'tables/table-si-g19.txt')
  cat('Table SI.G.19', '\n\n')
  as.data.frame(diermeierandstevenson1999.ph.tab.pretty3)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.19 supplemental information', '\n\n')
  diermeierandstevenson1999.ph.tab3
sink()

zphPlots(cox.fit = diermeierandstevenson1999.cox3,
  var.labs = diermeierandstevenson1999.var.labs,
  var.labs.pretty = diermeierandstevenson1999.var.labs.pretty,
  directory = 'figures/supplemental/diermeierandstevenson1999/model3')

star.tab.temp4 <- matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab4)),
  ncol = 4)

for ( i in 1:nrow(diermeierandstevenson1999.ph.tab4) ) {
  for ( j in 1:4 ) {
    ifelse ( diermeierandstevenson1999.ph.tab4[i, p.val.cols[j]] < .05,
      star.tab.temp4[i, j] <- '*', star.tab.temp4[i, j] <- ' ' )
  }
}
diermeierandstevenson1999.ph.tab.pretty4 <-
  matrix(rep(NA, 4 * nrow(diermeierandstevenson1999.ph.tab4)), ncol = 4)
rownames(diermeierandstevenson1999.ph.tab.pretty4) <-
  c(diermeierandstevenson1999.var.labs.pretty, 'GLOBAL')
colnames(diermeierandstevenson1999.ph.tab.pretty4) <- cols
for ( i in 1:nrow(diermeierandstevenson1999.ph.tab4) - 1 ) {
  for ( j in 1:4 ) {
    diermeierandstevenson1999.ph.tab.pretty4[i, j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab4[i, rho.cols[j]]),
      star.tab.temp4[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  diermeierandstevenson1999.ph.tab.pretty4[
    nrow(diermeierandstevenson1999.ph.tab4), j] <-
      paste(sprintf("%.3f", diermeierandstevenson1999.ph.tab4[
        nrow(diermeierandstevenson1999.ph.tab4), chi.sq.cols[j]]),
        star.tab.temp4[nrow(diermeierandstevenson1999.ph.tab4), j], sep = '')
}

sink(file = 'tables/table-si-g20.txt')
  cat('Table SI.G.20', '\n\n')
  as.data.frame(diermeierandstevenson1999.ph.tab.pretty4)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.20 supplemental information', '\n\n')
  diermeierandstevenson1999.ph.tab4
sink()

zphPlot1(cox.fit = diermeierandstevenson1999.cox4, var.lab = 'cmajg',
  var.lab.pretty = 'Majority Status', directory = 'figures',
  fig.name = 'figure-si-g24')
zphPlot1(cox.fit = diermeierandstevenson1999.cox1, var.lab = 'invest',
  var.lab.pretty = 'Investiture Vote', directory = 'figures',
  fig.name = 'figure-si-g25')

zphPlots(cox.fit = diermeierandstevenson1999.cox4,
  var.labs = diermeierandstevenson1999.var.labs,
  var.labs.pretty = diermeierandstevenson1999.var.labs.pretty,
  directory = 'figures/supplemental/diermeierandstevenson1999/model4')
################################################################################


################################################################################
##
## Golder, Matt, Sona N. Golder, and David A. Siegel. 2012. "Modeling the
##   Institutional Foundation of Parliamentary Government Formation." Journal of
##   Politics 74(2): 427-445.
##
##     - results identical to original paper
##

golderetal2012 <-
  read.csv('data/nonpublic/golderetal2012/GGS_JOP_GovForm_Data.csv')
names(golderetal2012)[74] <- 'Prop_Single.Party_Majority'
names(golderetal2012)[75] <- 'Prop_Single.Party_Minority'
names(golderetal2012)[76] <- 'Prop_Coalition_Minority'
names(golderetal2012)[77] <- 'Prop_Minimal_Winning'
names(golderetal2012)[78] <- 'Prop_Surplus_Simple'
names(golderetal2012)[79] <- 'Prop_Unity'
names(golderetal2012)[2] <- 'beta_sim'
names(golderetal2012)[24] <- 'enpp'
names(golderetal2012)[25] <- 'diversity'
names(golderetal2012)[66] <- 'delay'

Prop_Surplus <- with(golderetal2012, Prop_Surplus_Simple + Prop_Unity)
golderetal2012 <- cbind(golderetal2012, Prop_Surplus)
psg <- with(golderetal2012, Proportion.Surplus.Govt + Proportion.Unity.Govt)
golderetal2012 <- cbind(golderetal2012, psg)
rm(Prop_Surplus)
rm(psg)

real1 <-
  read.csv('data/nonpublic/golderetal2012/GGS_JOP_GovForm_Data_real1.csv')
bw <- npudensbw(formula = ~enpp, dat = real1)
enpp_w <- npudens(bws = bw, newdata = golderetal2012)
enpp_weights <- enpp_w$dens
golderetal2012$enpp_weights <- enpp_weights
rm(enpp_weights)

golderetal2012$delay2 <- 1
golderetal2012[golderetal2012$delay == 100, ]$delay2 <- 0

#golderetal2012<-cbind(golderetal2012,delay2)
surv <- with(golderetal2012, Surv(delay, delay2))
golderetal2012.cox <- coxph(surv ~ beta_sim + numParties + enpp + diversity,
  data = golderetal2012, weights = enpp_weights)
summary(golderetal2012.cox)

golderetal2012.ph.tab <- zphTable(golderetal2012.cox, digits = 3)
golderetal2012.ph.tab
golderetal2012.ph.tab[, c(3, 6, 9, 12)]

golderetal2012.var.labs <- rownames(golderetal2012.ph.tab)
golderetal2012.var.labs.pretty <- c('Policy Weight', 'No. of Parties',
  'Eff. No. of Parties', 'Ideo. Diversity')

star.tab.temp <- matrix(rep(NA, 4 * nrow(golderetal2012.ph.tab)), ncol = 4)

for ( i in 1:nrow(golderetal2012.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( golderetal2012.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
golderetal2012.ph.tab.pretty <- matrix(rep(NA, 4 * nrow(golderetal2012.ph.tab)),
  ncol = 4)
rownames(golderetal2012.ph.tab.pretty) <- c(golderetal2012.var.labs.pretty,
  'GLOBAL')
colnames(golderetal2012.ph.tab.pretty) <- cols
for ( i in 1:nrow(golderetal2012.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    golderetal2012.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", golderetal2012.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  golderetal2012.ph.tab.pretty[nrow(golderetal2012.ph.tab), j] <-
    paste(sprintf("%.3f", golderetal2012.ph.tab[nrow(golderetal2012.ph.tab),
    chi.sq.cols[j]]), star.tab.temp[nrow(golderetal2012.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g21.txt')
  cat('Table SI.G.21', '\n\n')
  as.data.frame(golderetal2012.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.21 supplemental information', '\n\n')
  golderetal2012.ph.tab
sink()

zphPlots(cox.fit = golderetal2012.cox, var.labs = golderetal2012.var.labs,
  var.labs.pretty = golderetal2012.var.labs.pretty,
  directory = 'figures/supplemental/golderetal2012')
################################################################################


################################################################################
##
## Hartzell, Caroline, and Matthew Hoddie. 2003. "Institutionalizing Peace:
##   Power Sharing and Post-Civil War Conflict Management." American Journal of
##   Political Science 47(2): 318-332.
##
##     - Weibull model does not replicate; could be a result of aftreg
##       using a different parameterization than Stata's parameterization
##

hartzellandhoddie2003 <-
  read.csv('data/nonpublic/hartzellandhoddie2003/ajpsdata.csv')

surv <- with(hartzellandhoddie2003, Surv(peacedur, peacefail))

#weibull does not replicate
hartzellandhoddie2003.weib <- aftreg(surv ~ totalps + prt + condurl + ldeadper +
  thrdpart + systruc + conissue, dist = 'weibull', param = 'lifeAcc',
  data = hartzellandhoddie2003)
summary(hartzellandhoddie2003.weib)

hartzellandhoddie2003.cox <- coxph(surv ~ totalps + prt + condurl + ldeadper + 
  thrdpart + systruc + conissue, data = hartzellandhoddie2003)
summary(hartzellandhoddie2003.cox)

hartzellandhoddie2003.ph.tab <- zphTable(hartzellandhoddie2003.cox, digits = 3)
hartzellandhoddie2003.ph.tab
hartzellandhoddie2003.ph.tab[, c(3, 6, 9, 12)]

hartzellandhoddie2003.var.labs <- rownames(hartzellandhoddie2003.ph.tab)
hartzellandhoddie2003.var.labs.pretty <- c('Power-Sharing Institutions', 
  'Previous Regime Type', 'Conflict Duration', 'Conflict Intensity',
  'Third Party Enforcer', 'International System Structure', 'Conflict Issue')

star.tab.temp <- matrix(rep(NA, 4 * nrow(hartzellandhoddie2003.ph.tab)),
  ncol = 4)

for ( i in 1:nrow(hartzellandhoddie2003.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( hartzellandhoddie2003.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
hartzellandhoddie2003.ph.tab.pretty <-
  matrix(rep(NA, 4 * nrow(hartzellandhoddie2003.ph.tab)), ncol = 4)
rownames(hartzellandhoddie2003.ph.tab.pretty) <-
  c(hartzellandhoddie2003.var.labs.pretty, 'GLOBAL')
colnames(hartzellandhoddie2003.ph.tab.pretty) <- cols
for ( i in 1:nrow(hartzellandhoddie2003.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    hartzellandhoddie2003.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", hartzellandhoddie2003.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  hartzellandhoddie2003.ph.tab.pretty[nrow(hartzellandhoddie2003.ph.tab), j] <-
    paste(sprintf("%.3f", hartzellandhoddie2003.ph.tab[
      nrow(hartzellandhoddie2003.ph.tab), chi.sq.cols[j]]),
      star.tab.temp[nrow(hartzellandhoddie2003.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g22.txt')
  cat('Table SI.G.22', '\n\n')
  as.data.frame(hartzellandhoddie2003.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.22 supplemental information', '\n\n')
  hartzellandhoddie2003.ph.tab
sink()

zphPlot1(cox.fit = hartzellandhoddie2003.cox, var.lab = 'prt',
  var.lab.pretty = 'Previous Regime Type', directory = 'figures',
  fig.name = 'figure-si-g26')

zphPlots(cox.fit = hartzellandhoddie2003.cox, 
  var.labs = hartzellandhoddie2003.var.labs,
  var.labs.pretty = hartzellandhoddie2003.var.labs.pretty,
  directory = 'figures/supplemental/hartzellandhoddie2003')
################################################################################


################################################################################
##
## Leeds, Brett Ashley, and Burcu Savun. 2007. "Terminating Alliances: Why Do
##   States Abrogate Agreements?" Journal of Politics 69(4): 1118-1132.
##
##     - results identical to original paper
##

leedsandsavun2007 <-
  read.dta('data/nonpublic/leedsandsavun2007/LeedsSavunJOP2007.dta')

leedsandsavun2007 <- leedsandsavun2007[
  order(leedsandsavun2007$atopidphase, leedsandsavun2007$obyear), ]

leedsandsavun2007$t0 <- NA
for ( i in unique(leedsandsavun2007$atopidphase) ) {
  ifelse(length(leedsandsavun2007[
    leedsandsavun2007$atopidphase == i, ]$dur) > 1,
  leedsandsavun2007[leedsandsavun2007$atopidphase == i, ]$t0 <-
    with(leedsandsavun2007, c(0, leedsandsavun2007[atopidphase == i, ]$dur[
    1:(length(leedsandsavun2007[atopidphase == i, ]$dur) - 1)])),
  leedsandsavun2007[leedsandsavun2007$atopidphase == i, ]$t0 <- 0)
}

surv <- with(leedsandsavun2007, Surv(t0, dur, violate))

leedsandsavun2007.cox <- coxph(surv ~ capchangeeither + regchangeeither + 
  chTEeither30lt + alform + demever + asymmetry + nomicoop + treaty + milinst +
  cluster(atopidphase), data = leedsandsavun2007)
summary(leedsandsavun2007.cox)

leedsandsavun2007.ph.tab <- zphTable(leedsandsavun2007.cox, digits = 3)
leedsandsavun2007.ph.tab
leedsandsavun2007.ph.tab[, c(3, 6, 9, 12)]

leedsandsavun2007.var.labs <- rownames(leedsandsavun2007.ph.tab)
leedsandsavun2007.var.labs.pretty <- c('Change in International Power', 
  'Change in Political Institutions', 'Decrease in External Threat', 
  'Formation of New Outside Alliance', 'Democratic Members', 'Asymmetric Power', 
  'Agr. Incl. Non-Military Coop.', 'Treaty Requiring Ratification', 
  'Agr. Req. Peacetime Mil. Coord.')

star.tab.temp <- matrix(rep(NA, 4 * nrow(leedsandsavun2007.ph.tab)), ncol = 4)

for ( i in 1:nrow(leedsandsavun2007.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( leedsandsavun2007.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
leedsandsavun2007.ph.tab.pretty <-
  matrix(rep(NA, 4 * nrow(leedsandsavun2007.ph.tab)), ncol = 4)
rownames(leedsandsavun2007.ph.tab.pretty) <-
  c(leedsandsavun2007.var.labs.pretty, 'GLOBAL')
colnames(leedsandsavun2007.ph.tab.pretty) <- cols
for ( i in 1:nrow(leedsandsavun2007.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    leedsandsavun2007.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", leedsandsavun2007.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  leedsandsavun2007.ph.tab.pretty[nrow(leedsandsavun2007.ph.tab), j] <-
    paste(sprintf("%.3f", leedsandsavun2007.ph.tab[
      nrow(leedsandsavun2007.ph.tab), chi.sq.cols[j]]),
      star.tab.temp[nrow(leedsandsavun2007.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g23.txt')
  cat('Table SI.G.23', '\n\n')
  as.data.frame(leedsandsavun2007.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.23 supplemental information', '\n\n')
  leedsandsavun2007.ph.tab
sink()

zphPlots(cox.fit = leedsandsavun2007.cox, var.labs = leedsandsavun2007.var.labs,
  var.labs.pretty = leedsandsavun2007.var.labs.pretty,
  directory = 'figures/supplemental/leedsandsavun2007')
################################################################################


################################################################################
##
## Long, Andrew G., Timothy Nordstrom, and Kyeonghi Baek. 2007. "Allying for
##   Peace: Treaty Obligations and Conflict between Allies." Journal of Politics
##   69(4): 1103-1117.
##
##     - results identical to original paper
##

longetal2007 <- read.csv('data/nonpublic/longetal2007/duration_of_peace.csv')

longetal2007$t0 <- longetal2007$'X_t0'
longetal2007$t <- longetal2007$'X_t'

surv <- with(longetal2007, Surv(t0, t, buofmzmid))

longetal2007.cox <- coxph(surv ~ arbcom + organ + noagg + medarb + milinst + 
  relcap + border + jdem  + s_wt_glo + wartime + major, method = 'breslow',
  data = longetal2007)
summary(longetal2007.cox)

longetal2007.ph.tab <- zphTable(longetal2007.cox, digits = 3)
longetal2007.ph.tab
longetal2007.ph.tab[, c(3, 6, 9, 12)]

longetal2007.var.labs <- rownames(longetal2007.ph.tab)
longetal2007.var.labs.pretty <- c('Arbitration Commission', 'Organization',
  'Non-aggression', 'Mediate Disputes', 'Military Institutionalization',
  'Relative Capabilities', 'Shared Border', 'Joint Democracy', 'Similarity',
  'Wartime Alliance', 'Major Power')

star.tab.temp <- matrix(rep(NA, 4 * nrow(longetal2007.ph.tab)), ncol = 4)

for ( i in 1:nrow(longetal2007.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( longetal2007.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
longetal2007.ph.tab.pretty <- matrix(rep(NA, 4 * nrow(longetal2007.ph.tab)),
  ncol = 4)
rownames(longetal2007.ph.tab.pretty) <- c(longetal2007.var.labs.pretty,
  'GLOBAL')
colnames(longetal2007.ph.tab.pretty) <- cols
for ( i in 1:nrow(longetal2007.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    longetal2007.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", longetal2007.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  longetal2007.ph.tab.pretty[nrow(longetal2007.ph.tab), j] <-
    paste(sprintf("%.3f", longetal2007.ph.tab[nrow(longetal2007.ph.tab),
    chi.sq.cols[j]]), star.tab.temp[nrow(longetal2007.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g24.txt')
  cat('Table SI.G.24', '\n\n')
  as.data.frame(longetal2007.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.24 supplemental information', '\n\n')
  longetal2007.ph.tab
sink()

zphPlots(cox.fit = longetal2007.cox, var.labs = longetal2007.var.labs,
  var.labs.pretty = longetal2007.var.labs.pretty,
  directory = 'figures/supplemental/longetal2007')
################################################################################


################################################################################
##
## Lyall, Jason. 2010. "Are Coethnics More Effective Counterinsurgents? Evidence
##   from the Second Chechen War." American Political Science Review 104(1):
##   1-20.
##
##     - Weibull results approximately replicate
##

lyall2010 <- read.dta('data/nonpublic/lyall2010/RussChechMatched.dta')

lyall2010$fail2 <- lyall2010$fail
lyall2010[lyall2010$fail == 0.5, ]$fail2 <- 1

surv1 <- with(lyall2010, Surv(vildays, fail2))
lyall2010.weib1 <- survreg(surv1 ~ -1 + treat + frailty(vilno),
  dist = 'weibull', data = lyall2010)
lyall2010.weib2 <- survreg(surv1 ~ -1 + treat + lpop2000 + lelev + liso +
  lhistory + poverty + steald + pret + gar + tariq + killd + reb + tac +
  logtotal + month + year + groznyy + frailty(vilno), dist = 'weibull', 
  data = lyall2010)

surv2 <- with(lyall2010[lyall2010$groznyy != 1, ], Surv(vildays, fail2))
lyall2010.weib3 <- survreg(surv2 ~ -1 + treat + frailty(vilno),
  dist = 'weibull', data = lyall2010[lyall2010$groznyy != 1, ])
lyall2010.weib4 <- survreg(surv2 ~ -1 + treat + lpop2000 + lelev + liso +
  lhistory + poverty + steald + pret + gar + tariq + killd + reb + tac +
  logtotal + month + year + frailty(vilno), dist = 'weibull', 
  data = lyall2010[lyall2010$groznyy != 1, ])

summary(lyall2010.weib1)
summary(lyall2010.weib2)
summary(lyall2010.weib3)
summary(lyall2010.weib4)

lyall2010.cox1 <- coxph(surv1 ~ treat + frailty(vilno), data = lyall2010)
lyall2010.cox2 <- coxph(surv1 ~ treat + lpop2000 + lelev + liso +
  lhistory + poverty + steald + pret + gar + tariq + killd + reb + tac +
  logtotal + month + year + groznyy + frailty(vilno), data = lyall2010,
  control = coxph.control(eps = 1e-09, iter.max = 10000, outer.max = 1000))
lyall2010.cox3 <- coxph(surv2 ~ treat + frailty(vilno), 
  data = lyall2010[lyall2010$groznyy != 1, ])
lyall2010.cox4 <- coxph(surv2 ~ treat + lpop2000 + lelev + liso +
  lhistory + poverty + steald + pret + gar + tariq + killd + reb + tac +
  logtotal + month + year + frailty(vilno), 
  data = lyall2010[lyall2010$groznyy != 1, ],
  control = coxph.control(eps = 1e-09, iter.max = 100, outer.max = 100))

summary(lyall2010.cox1)
summary(lyall2010.cox2)
summary(lyall2010.cox3)
summary(lyall2010.cox4)

lyall2010.ph.tab1 <- zphTable(lyall2010.cox1, digits = 3)
lyall2010.ph.tab1
lyall2010.ph.tab1[, c(3, 6, 9, 12)]
lyall2010.ph.tab2 <- zphTable(lyall2010.cox2, digits = 3)
lyall2010.ph.tab2
lyall2010.ph.tab2[, c(3, 6, 9, 12)]
lyall2010.ph.tab3 <- zphTable(lyall2010.cox3, digits = 3)
lyall2010.ph.tab3
lyall2010.ph.tab3[, c(3, 6, 9, 12)]
lyall2010.ph.tab4 <- zphTable(lyall2010.cox4, digits = 3)
lyall2010.ph.tab4
lyall2010.ph.tab4[, c(3, 6, 9, 12)]

lyall2010.var.labs1 <- rownames(lyall2010.ph.tab1)
lyall2010.var.labs.pretty1 <- c('Treat')
lyall2010.var.labs2 <- rownames(lyall2010.ph.tab2)
lyall2010.var.labs.pretty2 <- c('Treat', 'Log(Population) in 2000', 
  'Log(Elevation)', 'Log(Isolation)', 'Log(History)', 'Poverty', 
  'Large-Scale Theft', 'Presweep Violence', 'Garrison', 'Tariqa', 'Killing', 
  'Rebel', 'TAC', 'Log(Total Abuse)', 'Month', 'Year', 'Groznyy')
lyall2010.var.labs3 <- rownames(lyall2010.ph.tab3)
lyall2010.var.labs.pretty3 <- c('Treat')
lyall2010.var.labs4 <- rownames(lyall2010.ph.tab4)
lyall2010.var.labs.pretty4 <- c('Treat', 'Log(Population) in 2000', 
  'Log(Elevation)', 'Log(Isolation)', 'Log(History)', 'Poverty', 
  'Large-Scale Theft', 'Presweep Violence', 'Garrison', 'Tariqa', 'Killing', 
  'Rebel', 'TAC', 'Log(Total Abuse)', 'Month', 'Year')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab1)), ncol = 4)

for ( i in 1:nrow(lyall2010.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( lyall2010.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
lyall2010.ph.tab.pretty1 <- matrix(rep(NA, 4 * (nrow(lyall2010.ph.tab1) + 1)),
  ncol = 4)
rownames(lyall2010.ph.tab.pretty1) <- c(lyall2010.var.labs.pretty1, 'GLOBAL')
colnames(lyall2010.ph.tab.pretty1) <- cols
for ( i in 1:nrow(lyall2010.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    lyall2010.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", lyall2010.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  lyall2010.ph.tab.pretty1[nrow(lyall2010.ph.tab1), j] <-
    paste(sprintf("%.3f", lyall2010.ph.tab1[nrow(lyall2010.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(lyall2010.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g25.txt')
  cat('Table SI.G.25', '\n\n')
  as.data.frame(lyall2010.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.25 supplemental information', '\n\n')
  lyall2010.ph.tab1
sink()

zphPlots(cox.fit = lyall2010.cox1, var.labs = lyall2010.var.labs1,
  var.labs.pretty = lyall2010.var.labs.pretty1,
  directory = 'figures/supplemental/lyall2010/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab2)), ncol = 4)

for ( i in 1:nrow(lyall2010.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( lyall2010.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
lyall2010.ph.tab.pretty2 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab2)),
  ncol = 4)
rownames(lyall2010.ph.tab.pretty2) <- c(lyall2010.var.labs.pretty2, 'GLOBAL')
colnames(lyall2010.ph.tab.pretty2) <- cols
for ( i in 1:nrow(lyall2010.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    lyall2010.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", lyall2010.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  lyall2010.ph.tab.pretty2[nrow(lyall2010.ph.tab2), j] <-
    paste(sprintf("%.3f", lyall2010.ph.tab2[nrow(lyall2010.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(lyall2010.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g26.txt')
  cat('Table SI.G.26', '\n\n')
  as.data.frame(lyall2010.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.26 supplemental information', '\n\n')
  lyall2010.ph.tab2
sink()

zphPlot1(cox.fit = lyall2010.cox2, var.lab = 'lhistory',
  var.lab.pretty = 'Log(History)', directory = 'figures',
  fig.name = 'figure-si-g27')
zphPlot1(cox.fit = lyall2010.cox2, var.lab = 'steald',
  var.lab.pretty = 'Large-Scale Theft', directory = 'figures',
  fig.name = 'figure-si-g28')
zphPlot1(cox.fit = lyall2010.cox2, var.lab = 'gar',
  var.lab.pretty = 'Garrison', directory = 'figures',
  fig.name = 'figure-si-g29')
zphPlot1(cox.fit = lyall2010.cox2, var.lab = 'logtotal',
  var.lab.pretty = 'Log(Total Abuse)', directory = 'figures',
  fig.name = 'figure-si-g30')
zphPlot1(cox.fit = lyall2010.cox2, var.lab = 'year',
  var.lab.pretty = 'Year', directory = 'figures',
  fig.name = 'figure-si-g31')

zphPlots(cox.fit = lyall2010.cox2, var.labs = lyall2010.var.labs2,
  var.labs.pretty = lyall2010.var.labs.pretty2,
  directory = 'figures/supplemental/lyall2010/model2')

star.tab.temp3 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab3)), ncol = 4)

for ( i in 1:nrow(lyall2010.ph.tab3) ) {
  for ( j in 1:4 ) {
    ifelse ( lyall2010.ph.tab3[i, p.val.cols[j]] < .05,
      star.tab.temp3[i, j] <- '*', star.tab.temp3[i, j] <- ' ' )
  }
}
lyall2010.ph.tab.pretty3 <- matrix(rep(NA, 4 * (nrow(lyall2010.ph.tab3) + 1)),
  ncol = 4)
rownames(lyall2010.ph.tab.pretty3) <- c(lyall2010.var.labs.pretty3, 'GLOBAL')
colnames(lyall2010.ph.tab.pretty3) <- cols
for ( i in 1:nrow(lyall2010.ph.tab3) - 1 ) {
  for ( j in 1:4 ) {
    lyall2010.ph.tab.pretty3[i, j] <-
      paste(sprintf("%.3f", lyall2010.ph.tab3[i, rho.cols[j]]),
      star.tab.temp3[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  lyall2010.ph.tab.pretty3[nrow(lyall2010.ph.tab3), j] <-
    paste(sprintf("%.3f", lyall2010.ph.tab3[nrow(lyall2010.ph.tab3),
    chi.sq.cols[j]]), star.tab.temp3[nrow(lyall2010.ph.tab3), j], sep = '')
}

sink(file = 'tables/table-si-g27.txt')
  cat('Table SI.G.27', '\n\n')
  as.data.frame(lyall2010.ph.tab.pretty3)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.27 supplemental information', '\n\n')
  lyall2010.ph.tab3
sink()

zphPlots(cox.fit = lyall2010.cox3, var.labs = lyall2010.var.labs3,
  var.labs.pretty = lyall2010.var.labs.pretty3,
  directory = 'figures/supplemental/lyall2010/model3')

star.tab.temp4 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab4)), ncol = 4)

for ( i in 1:nrow(lyall2010.ph.tab4) ) {
  for ( j in 1:4 ) {
    ifelse ( lyall2010.ph.tab4[i, p.val.cols[j]] < .05,
      star.tab.temp4[i, j] <- '*', star.tab.temp4[i, j] <- ' ' )
  }
}
lyall2010.ph.tab.pretty4 <- matrix(rep(NA, 4 * nrow(lyall2010.ph.tab4)),
  ncol = 4)
rownames(lyall2010.ph.tab.pretty4) <- c(lyall2010.var.labs.pretty4, 'GLOBAL')
colnames(lyall2010.ph.tab.pretty4) <- cols
for ( i in 1:nrow(lyall2010.ph.tab4) - 1 ) {
  for ( j in 1:4 ) {
    lyall2010.ph.tab.pretty4[i, j] <-
      paste(sprintf("%.3f", lyall2010.ph.tab4[i, rho.cols[j]]),
      star.tab.temp4[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  lyall2010.ph.tab.pretty4[nrow(lyall2010.ph.tab4), j] <-
    paste(sprintf("%.3f", lyall2010.ph.tab4[nrow(lyall2010.ph.tab4),
    chi.sq.cols[j]]), star.tab.temp4[nrow(lyall2010.ph.tab4), j], sep = '')
}

sink(file = 'tables/table-si-g28.txt')
  cat('Table SI.G.28', '\n\n')
  as.data.frame(lyall2010.ph.tab.pretty4)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.28 supplemental information', '\n\n')
  lyall2010.ph.tab4
sink()

zphPlot1(cox.fit = lyall2010.cox4, var.lab = 'lelev',
  var.lab.pretty = 'Log(Elevation)', directory = 'figures',
  fig.name = 'figure-si-g32')
zphPlot1(cox.fit = lyall2010.cox4, var.lab = 'steald',
  var.lab.pretty = 'Large-Scale Theft', directory = 'figures',
  fig.name = 'figure-si-g33')
zphPlot1(cox.fit = lyall2010.cox4, var.lab = 'logtotal',
  var.lab.pretty = 'Log(Total Abuse)', directory = 'figures',
  fig.name = 'figure-si-g34')
zphPlot1(cox.fit = lyall2010.cox4, var.lab = 'month',
  var.lab.pretty = 'Month', directory = 'figures',
  fig.name = 'figure-si-g35')

zphPlots(cox.fit = lyall2010.cox4, var.labs = lyall2010.var.labs4,
  var.labs.pretty = lyall2010.var.labs.pretty4,
  directory = 'figures/supplemental/lyall2010/model4')
################################################################################


################################################################################
##
## Maeda, Ko. 2010. "Two Modes of Democratic Breakdown: A Competing Risk
##   Analysis of Democratic Durability." Journal of Politics 72(4): 1129-1143.
##
##     - results identical to original paper
##

maeda2010 <- read.dta('data/nonpublic/maeda2010/Maeda2010JOP_TwoModes.dta')

maeda2010$t0 <- NA
for ( i in unique(maeda2010$demid) ) {
  ifelse(length(maeda2010[maeda2010$demid == i, ]$demmonth) > 1,
    maeda2010[maeda2010$demid == i, ]$t0 <-
      with(maeda2010, c(0, maeda2010[demid == i, ]$demmonth[
      1:(length(maeda2010[demid == i, ]$demmonth) - 1)])),
    maeda2010[maeda2010$demid == i, ]$t0 <- 0)
}

maeda2010$demend2 <- maeda2010$demend
maeda2010[maeda2010$demend == 9, ]$demend2 <- 0
maeda2010$endmode2 <- maeda2010$endmode
maeda2010[maeda2010$endmode == 2, ]$endmode2 <- 0
maeda2010$endmode3 <- 0
maeda2010[maeda2010$endmode == 2, ]$endmode3 <- 1

surv1 <- with(maeda2010, Surv(t0, demmonth, demend2))
maeda2010.cox1 <- coxph(surv1 ~ dev + gro + presi + mixed + majgovm + ethnic + 
  ope + urb + postcw + impose + prereg_ind + prereg_mil + regiondem +
  cluster(country), data = maeda2010)

surv2 <- with(maeda2010, Surv(t0, demmonth, endmode2))
maeda2010.cox2 <- coxph(surv2 ~ dev + gro + presi + mixed + majgovm + ethnic + 
  ope + urb + postcw + impose + prereg_ind + prereg_mil + regiondem +
  cluster(country), data = maeda2010)

surv3 <- with(maeda2010, Surv(t0, demmonth, endmode3))
maeda2010.cox3 <- coxph(surv3 ~ dev + gro + presi + mixed + majgovm + ethnic + 
  ope + urb + postcw + impose + prereg_ind + prereg_mil + regiondem +
  cluster(country), data = maeda2010)

summary(maeda2010.cox1)
summary(maeda2010.cox2)
summary(maeda2010.cox3)

maeda2010.ph.tab1 <- zphTable(maeda2010.cox1, digits = 3)
maeda2010.ph.tab2 <- zphTable(maeda2010.cox2, digits = 3)
maeda2010.ph.tab3 <- zphTable(maeda2010.cox3, digits = 3)

maeda2010.ph.tab1
maeda2010.ph.tab1[, c(3, 6, 9, 12)]
maeda2010.ph.tab2
maeda2010.ph.tab2[, c(3, 6, 9, 12)]
maeda2010.ph.tab3
maeda2010.ph.tab3[, c(3, 6, 9, 12)]

maeda2010.var.labs <- rownames(maeda2010.ph.tab1)
maeda2010.var.labs.pretty <- c('Development', 'Growth', 'Presidential System',
  'Mixed System', 'Majority Government', 'Ethnic Fragmentation', 
  'Trade Openness', 'Urbanization', 'Post-Cold War Era', 'Imposed Polity',
  'Colony', 'Military', 'Regional Democracy Level')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab1)), ncol = 4)

for ( i in 1:nrow(maeda2010.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( maeda2010.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
maeda2010.ph.tab.pretty1 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab1)),
  ncol = 4)
rownames(maeda2010.ph.tab.pretty1) <- c(maeda2010.var.labs.pretty, 'GLOBAL')
colnames(maeda2010.ph.tab.pretty1) <- cols
for ( i in 1:nrow(maeda2010.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    maeda2010.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", maeda2010.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  maeda2010.ph.tab.pretty1[nrow(maeda2010.ph.tab1), j] <-
    paste(sprintf("%.3f", maeda2010.ph.tab1[nrow(maeda2010.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(maeda2010.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g29.txt')
  cat('Table SI.G.29', '\n\n')
  as.data.frame(maeda2010.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.29 supplemental information', '\n\n')
  maeda2010.ph.tab1
sink()

zphPlot1(cox.fit = maeda2010.cox1, var.lab = 'presi',
  var.lab.pretty = 'Presidential System', directory = 'figures',
  fig.name = 'figure-si-g36')
zphPlot1(cox.fit = maeda2010.cox1, var.lab = 'prereg_ind',
  var.lab.pretty = 'Colony', directory = 'figures',
  fig.name = 'figure-si-g37')

zphPlots(cox.fit = maeda2010.cox1, var.labs = maeda2010.var.labs,
  var.labs.pretty = maeda2010.var.labs.pretty,
  directory = 'figures/supplemental/maeda2010/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab2)), ncol = 4)

for ( i in 1:nrow(maeda2010.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( maeda2010.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
maeda2010.ph.tab.pretty2 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab2)),
  ncol = 4)
rownames(maeda2010.ph.tab.pretty2) <- c(maeda2010.var.labs.pretty, 'GLOBAL')
colnames(maeda2010.ph.tab.pretty2) <- cols
for ( i in 1:nrow(maeda2010.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    maeda2010.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", maeda2010.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  maeda2010.ph.tab.pretty2[nrow(maeda2010.ph.tab2), j] <-
    paste(sprintf("%.3f", maeda2010.ph.tab2[nrow(maeda2010.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(maeda2010.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g30.txt')
  cat('Table SI.G.30', '\n\n')
  as.data.frame(maeda2010.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.30 supplemental information', '\n\n')
  maeda2010.ph.tab2
sink()

zphPlot1(cox.fit = maeda2010.cox2, var.lab = 'majgovm',
  var.lab.pretty = 'Majority Government', directory = 'figures',
  fig.name = 'figure-si-g38')
zphPlot1(cox.fit = maeda2010.cox2, var.lab = 'regiondem',
  var.lab.pretty = 'Regional Democracy Level', directory = 'figures',
  fig.name = 'figure-si-g39')

zphPlots(cox.fit = maeda2010.cox2, var.labs = maeda2010.var.labs,
  var.labs.pretty = maeda2010.var.labs.pretty,
  directory = 'figures/supplemental/maeda2010/model2')

star.tab.temp3 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab3)), ncol = 4)

for ( i in 1:nrow(maeda2010.ph.tab3) ) {
  for ( j in 1:4 ) {
    ifelse ( maeda2010.ph.tab3[i, p.val.cols[j]] < .05,
      star.tab.temp3[i, j] <- '*', star.tab.temp3[i, j] <- ' ' )
  }
}
maeda2010.ph.tab.pretty3 <- matrix(rep(NA, 4 * nrow(maeda2010.ph.tab3)),
  ncol = 4)
rownames(maeda2010.ph.tab.pretty3) <- c(maeda2010.var.labs.pretty, 'GLOBAL')
colnames(maeda2010.ph.tab.pretty3) <- cols
for ( i in 1:nrow(maeda2010.ph.tab3) - 1 ) {
  for ( j in 1:4 ) {
    maeda2010.ph.tab.pretty3[i, j] <-
      paste(sprintf("%.3f", maeda2010.ph.tab3[i, rho.cols[j]]),
      star.tab.temp3[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  maeda2010.ph.tab.pretty3[nrow(maeda2010.ph.tab3), j] <-
    paste(sprintf("%.3f", maeda2010.ph.tab3[nrow(maeda2010.ph.tab3),
    chi.sq.cols[j]]), star.tab.temp3[nrow(maeda2010.ph.tab3), j], sep = '')
}

sink(file = 'tables/table-si-g31.txt')
  cat('Table SI.G.31', '\n\n')
  as.data.frame(maeda2010.ph.tab.pretty3)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.31 supplemental information', '\n\n')
  maeda2010.ph.tab3
sink()

zphPlot1(cox.fit = maeda2010.cox3, var.lab = 'dev',
  var.lab.pretty = 'Development', directory = 'figures',
  fig.name = 'figure-si-g40')
zphPlot1(cox.fit = maeda2010.cox3, var.lab = 'presi',
  var.lab.pretty = 'Presidential System', directory = 'figures',
  fig.name = 'figure-si-g41')
zphPlot1(cox.fit = maeda2010.cox3, var.lab = 'impose',
  var.lab.pretty = 'Imposed Polity', directory = 'figures',
  fig.name = 'figure-si-g42')

zphPlots(cox.fit = maeda2010.cox3, var.labs = maeda2010.var.labs,
  var.labs.pretty = maeda2010.var.labs.pretty,
  directory = 'figures/supplemental/maeda2010/model3')
################################################################################


################################################################################
##
## Martin, Lanny W. 2004. "The Government Agenda in Parliamentary Democracies."
##   American Journal of Political Science 48(3): 445-461.
##
##     - results identical to original paper
##

martin2004 <- read.dta('data/nonpublic/martin2004/agenda6.dta')

martin2004$fail <- 1

surv <- with(martin2004, Surv(durat, fail))
martin2004.cox1 <- coxph(surv ~ cbcopsibw + wcabdivbw + opcopsibw + wopdivbw + 
  dim2 + dim3 + dim4 + dim5 + dim6 + dim7 + dim8 + germany + belgium + luxem, 
  ties = 'breslow', data = martin2004)
martin2004.cox2 <- coxph(surv ~ cbcopsibw + ccbcopsibw + wcabdivbw + 
  cwcabdivbw + opcopsibw + wopdivbw + dim2 + dim3 + dim4 + dim5 + dim6 + dim7 +
  dim8 + germany + belgium + luxem, ties = 'breslow', data = martin2004)
summary(martin2004.cox1)
summary(martin2004.cox2)

martin2004.ph.tab1 <- zphTable(martin2004.cox1, digits = 3)
martin2004.ph.tab2 <- zphTable(martin2004.cox2, digits = 3)

martin2004.ph.tab1
martin2004.ph.tab1[, c(3, 6, 9, 12)]
martin2004.ph.tab2
martin2004.ph.tab2[, c(3, 6, 9, 12)]

martin2004.var.labs1 <- rownames(martin2004.ph.tab1)
martin2004.var.labs.pretty1 <- c('Government Issue Saliency', 
  'Government Issue Divisiveness', 'Opposition Issue Saliency', 
  'Opposition Issue Divisiveness', 'Foreign Policy', 'Industrial Policy',
  'Social Policy', 'Clerical Policy', 'Agricultural Policy', 'Regional Policy',
  'Environmental Policy', 'Germany', 'Belgium', 'Luxembourg')
martin2004.var.labs2 <- rownames(martin2004.ph.tab2)
martin2004.var.labs.pretty2 <- c('Gov\'t Issue Saliency',
  'Gov\'t Iss. Sal.*ln(CIEP)', 'Gov\'t Iss. Divis.',
  'Gov\'t Iss. Div.*ln(CIEP)', 'Opposition Iss. Sal.', 
  'Opp. Iss. Divis.', 'Foreign Policy', 'Industrial Policy',
  'Social Policy', 'Clerical Policy', 'Agricult. Policy', 'Regional Policy',
  'Environ. Policy', 'Germany', 'Belgium', 'Luxembourg')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab1)), ncol = 4)

for ( i in 1:nrow(martin2004.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( martin2004.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
martin2004.ph.tab.pretty1 <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab1)),
  ncol = 4)
rownames(martin2004.ph.tab.pretty1) <- c(martin2004.var.labs.pretty1, 'GLOBAL')
colnames(martin2004.ph.tab.pretty1) <- cols
for ( i in 1:nrow(martin2004.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    martin2004.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", martin2004.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  martin2004.ph.tab.pretty1[nrow(martin2004.ph.tab1), j] <-
    paste(sprintf("%.3f", martin2004.ph.tab1[nrow(martin2004.ph.tab1),
    chi.sq.cols[j]]), star.tab.temp1[nrow(martin2004.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g32.txt')
  cat('Table SI.G.32', '\n\n')
  as.data.frame(martin2004.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.32 supplemental information', '\n\n')
  martin2004.ph.tab1
sink()

zphPlot1(cox.fit = martin2004.cox1, var.lab = 'cbcopsibw',
  var.lab.pretty = 'Government Issue Saliency', directory = 'figures',
  fig.name = 'figure-si-g43')
zphPlot1(cox.fit = martin2004.cox1, var.lab = 'dim3',
  var.lab.pretty = 'Industrial Policy', directory = 'figures',
  fig.name = 'figure-si-g44')
zphPlot1(cox.fit = martin2004.cox1, var.lab = 'dim4',
  var.lab.pretty = 'Social Policy', directory = 'figures',
  fig.name = 'figure-si-g45')
zphPlot1(cox.fit = martin2004.cox1, var.lab = 'dim8',
  var.lab.pretty = 'Environmental Policy', directory = 'figures',
  fig.name = 'figure-si-g46')
zphPlot1(cox.fit = martin2004.cox1, var.lab = 'luxem',
  var.lab.pretty = 'Luxembourg', directory = 'figures',
  fig.name = 'figure-si-g47')

zphPlots(cox.fit = martin2004.cox1, var.labs = martin2004.var.labs1,
  var.labs.pretty = martin2004.var.labs.pretty1,
  directory = 'figures/supplemental/martin2004/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab2)), ncol = 4)

for ( i in 1:nrow(martin2004.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( martin2004.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
martin2004.ph.tab.pretty2 <- matrix(rep(NA, 4 * nrow(martin2004.ph.tab2)),
  ncol = 4)
rownames(martin2004.ph.tab.pretty2) <- c(martin2004.var.labs.pretty2, 'GLOBAL')
colnames(martin2004.ph.tab.pretty2) <- cols
for ( i in 1:nrow(martin2004.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    martin2004.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", martin2004.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  martin2004.ph.tab.pretty2[nrow(martin2004.ph.tab2), j] <-
    paste(sprintf("%.3f", martin2004.ph.tab2[nrow(martin2004.ph.tab2),
    chi.sq.cols[j]]), star.tab.temp2[nrow(martin2004.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g33.txt')
  cat('Table SI.G.33', '\n\n')
  as.data.frame(martin2004.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.33 supplemental information', '\n\n')
  martin2004.ph.tab2
sink()

zphPlot1(cox.fit = martin2004.cox2, var.lab = 'wcabdivbw',
  var.lab.pretty = 'Gov\'t Iss. Divis.', directory = 'figures',
  fig.name = 'figure-si-g51')
zphPlot1(cox.fit = martin2004.cox2, var.lab = 'cwcabdivbw',
  var.lab.pretty = 'Gov\'t Iss. Div.*ln(CIEP)', directory = 'figures',
  fig.name = 'figure-si-g49')
zphPlot1(cox.fit = martin2004.cox2, var.lab = 'dim2',
  var.lab.pretty = 'Foreign Policy', directory = 'figures',
  fig.name = 'figure-si-g50')

zphPlots(cox.fit = martin2004.cox2, var.labs = martin2004.var.labs2,
  var.labs.pretty = martin2004.var.labs.pretty2,
  directory = 'figures/supplemental/martin2004/model2')
################################################################################


################################################################################
##
## Martin, Lanny W, and Georg Vanberg. 2004. "Policing the Bargain: Coalition 
##   Government and Parliamentary Security." American Journal of Political 
##   Science 48(1): 13-27.
##
##     - Weibull results approximately replicate in R
##     - Cox results identical to results from authors' replication materials
##

martinandvanberg2004 <-
  read.dta('data/nonpublic/martinandvanberg2004/MartinVanbergAJPSreplication.dta')

surv <- with(martinandvanberg2004, Surv(lengthlow, censorlow))
martinandvanberg2004.weib1 <- aftreg(surv ~ wdivsal + wcoalimp + oppdivsal + 
  woppimp, dist = 'weibull', param = 'lifeAcc', data = martinandvanberg2004)
martinandvanberg2004.weib2 <- aftreg(surv ~ wdivsal + wcoalimp + oppdivsal +
  woppimp + dimension2 + dimension3 + dimension4 + dimension6 + dimension7 + 
  dimension8, dist = 'weibull', param = 'lifeAcc', data = martinandvanberg2004)

summary(martinandvanberg2004.weib1)
summary(martinandvanberg2004.weib2)

martinandvanberg2004.cox1 <- coxph(surv ~ wdivsal + wcoalimp + oppdivsal +
  woppimp, method = 'breslow', data = martinandvanberg2004)
martinandvanberg2004.cox2 <- coxph(surv ~ wdivsal + wcoalimp + oppdivsal +
  woppimp + dimension2 + dimension3 + dimension4 + dimension6 + dimension7 +
  dimension8, method = 'breslow', data = martinandvanberg2004)

summary(martinandvanberg2004.cox1)
summary(martinandvanberg2004.cox2)

martinandvanberg2004.ph.tab1 <- zphTable(martinandvanberg2004.cox1, digits = 3)
martinandvanberg2004.ph.tab2 <- zphTable(martinandvanberg2004.cox2, digits = 3)

martinandvanberg2004.ph.tab1
martinandvanberg2004.ph.tab1[, c(3, 6, 9, 12)]
martinandvanberg2004.ph.tab2
martinandvanberg2004.ph.tab2[, c(3, 6, 9, 12)]

martinandvanberg2004.var.labs1 <- rownames(martinandvanberg2004.ph.tab1)
martinandvanberg2004.var.labs.pretty1 <- c('Government Issue Divisiveness',
  'Government Issue Saliency', 'Opposition Issue Divisiveness',
  'Opposition Issue Saliency')
martinandvanberg2004.var.labs2 <- rownames(martinandvanberg2004.ph.tab2)
martinandvanberg2004.var.labs.pretty2 <- c('Government Issue Divisiveness',
  'Government Issue Saliency', 'Opposition Issue Divisiveness',
  'Opposition Issue Saliency', 'Foreign Policy', 'Industrial Policy',
  'Social Policy', 'Agricultural Policy', 'Regional Policy', 
  'Environmental Policy')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(martinandvanberg2004.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(martinandvanberg2004.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( martinandvanberg2004.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
martinandvanberg2004.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(martinandvanberg2004.ph.tab1)), ncol = 4)
rownames(martinandvanberg2004.ph.tab.pretty1) <-
  c(martinandvanberg2004.var.labs.pretty1, 'GLOBAL')
colnames(martinandvanberg2004.ph.tab.pretty1) <- cols
for ( i in 1:nrow(martinandvanberg2004.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    martinandvanberg2004.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", martinandvanberg2004.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  martinandvanberg2004.ph.tab.pretty1[nrow(martinandvanberg2004.ph.tab1), j] <-
    paste(sprintf("%.3f", martinandvanberg2004.ph.tab1[
      nrow(martinandvanberg2004.ph.tab1), chi.sq.cols[j]]),
    star.tab.temp1[nrow(martinandvanberg2004.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g34.txt')
  cat('Table SI.G.34', '\n\n')
  as.data.frame(martinandvanberg2004.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.34 supplemental information', '\n\n')
  martinandvanberg2004.ph.tab1
sink()

zphPlots(cox.fit = martinandvanberg2004.cox1, 
  var.labs = martinandvanberg2004.var.labs1,
  var.labs.pretty = martinandvanberg2004.var.labs.pretty1,
  directory = 'figures/supplemental/martinandvanberg2004/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(martinandvanberg2004.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(martinandvanberg2004.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( martinandvanberg2004.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
martinandvanberg2004.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(martinandvanberg2004.ph.tab2)), ncol = 4)
rownames(martinandvanberg2004.ph.tab.pretty2) <-
  c(martinandvanberg2004.var.labs.pretty2, 'GLOBAL')
colnames(martinandvanberg2004.ph.tab.pretty2) <- cols
for ( i in 1:nrow(martinandvanberg2004.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    martinandvanberg2004.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", martinandvanberg2004.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  martinandvanberg2004.ph.tab.pretty2[nrow(martinandvanberg2004.ph.tab2), j] <-
    paste(sprintf("%.3f", martinandvanberg2004.ph.tab2[
      nrow(martinandvanberg2004.ph.tab2), chi.sq.cols[j]]),
      star.tab.temp2[nrow(martinandvanberg2004.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g35.txt')
  cat('Table SI.G.35', '\n\n')
  as.data.frame(martinandvanberg2004.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.35 supplemental information', '\n\n')
  martinandvanberg2004.ph.tab2
sink()

zphPlot1(cox.fit = martinandvanberg2004.cox2, var.lab = 'dimension6',
  var.lab.pretty = 'Agricultural Policy', directory = 'figures',
  fig.name = 'figure-si-g51')

zphPlots(cox.fit = martinandvanberg2004.cox2,
  var.labs = martinandvanberg2004.var.labs2,
  var.labs.pretty = martinandvanberg2004.var.labs.pretty2,
  directory = 'figures/supplemental/martinandvanberg2004/model2')
################################################################################


################################################################################
##
## Mattes, Michaela, and Burcu Savun. 2010. "Information, Agreement Design, and
##   the Durability of Civil War Settlements." American Journal of Political
##   Science 54(2): 511-524.
##

mattesandsavun2010 <- read.dta('data/nonpublic/mattesandsavun2010/Rep.dta')

surv <- with(mattesandsavun2010, Surv(peacedur, peacefail))
mattesandsavun2010.cox <- coxph(surv ~ infoindex + psindex + guarantee +
  lifeexphh + hhpolity5 + issue + lncondeaths + lndur + cluster(ccode),
  data = mattesandsavun2010)
summary(mattesandsavun2010.cox)

mattesandsavun2010.ph.tab <- zphTable(mattesandsavun2010.cox, digits = 3)
mattesandsavun2010.ph.tab
mattesandsavun2010.ph.tab[, c(3, 6, 9, 12)]

mattesandsavun2010.var.labs <- rownames(mattesandsavun2010.ph.tab)
mattesandsavun2010.var.labs.pretty <- c('Uncert.-Reducing Prov.',
  'Power-Sharing Provisions', 'Third-Party Guarantees', 'Life Expectancy',
  'Previous Level of Democ.', 'Ethnic Issue', 'Costs of Previous War',
  'Duration of Previous War')

star.tab.temp <- matrix(rep(NA, 4 * nrow(mattesandsavun2010.ph.tab)), ncol = 4)

for ( i in 1:nrow(mattesandsavun2010.ph.tab) ) {
  for ( j in 1:4 ) {
    ifelse ( mattesandsavun2010.ph.tab[i, p.val.cols[j]] < .05,
      star.tab.temp[i, j] <- '*', star.tab.temp[i, j] <- ' ' )
  }
}
mattesandsavun2010.ph.tab.pretty <-
  matrix(rep(NA, 4 * nrow(mattesandsavun2010.ph.tab)), ncol = 4)
rownames(mattesandsavun2010.ph.tab.pretty) <-
  c(mattesandsavun2010.var.labs.pretty, 'GLOBAL')
colnames(mattesandsavun2010.ph.tab.pretty) <- cols
for ( i in 1:nrow(mattesandsavun2010.ph.tab) - 1 ) {
  for ( j in 1:4 ) {
    mattesandsavun2010.ph.tab.pretty[i, j] <-
      paste(sprintf("%.3f", mattesandsavun2010.ph.tab[i, rho.cols[j]]),
      star.tab.temp[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  mattesandsavun2010.ph.tab.pretty[nrow(mattesandsavun2010.ph.tab), j] <-
    paste(sprintf("%.3f", mattesandsavun2010.ph.tab[
      nrow(mattesandsavun2010.ph.tab), chi.sq.cols[j]]),
      star.tab.temp[nrow(mattesandsavun2010.ph.tab), j], sep = '')
}

sink(file = 'tables/table-si-g36.txt')
  cat('Table SI.G.36', '\n\n')
  as.data.frame(mattesandsavun2010.ph.tab.pretty)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.36 supplemental information', '\n\n')
  mattesandsavun2010.ph.tab
sink()

zphPlot1(cox.fit = mattesandsavun2010.cox, var.lab = 'lifeexphh',
  var.lab.pretty = 'Life Expectancy', directory = 'figures',
  fig.name = 'figure-si-g52')
zphPlot1(cox.fit = mattesandsavun2010.cox, var.lab = 'lncondeaths',
  var.lab.pretty = 'Costs of Previous War', directory = 'figures',
  fig.name = 'figure-si-g53')
zphPlot1(cox.fit = mattesandsavun2010.cox, var.lab = 'lndur',
  var.lab.pretty = 'Duration of Previous War', directory = 'figures',
  fig.name = 'figure-si-g54')

zphPlots(cox.fit = mattesandsavun2010.cox,
  var.labs = mattesandsavun2010.var.labs,
  var.labs.pretty = mattesandsavun2010.var.labs.pretty,
  directory = 'figures/supplemental/mattesandsavun2010/')
################################################################################


################################################################################
##
## Senese, Paul D., and Stephen L. Quackenbush. 2003. "Sowing the Seeds of
##   Conflict: The Effect of Dispute Settlements on Durations of Peace." Journal
##   of Politics 65(3): 696-717.
##
##     - results identical to original paper
##

seneseandquackenbush2003 <-
  read.dta('data/nonpublic/seneseandquackenbush2003/S&Q JOP 2003 replication data.dta')

seneseandquackenbush2003$t0 <- NA
for ( i in unique(seneseandquackenbush2003$case2) ) {
  ifelse(length(seneseandquackenbush2003[seneseandquackenbush2003$case2 == i, 
    ]$duratmo) > 1, 
    seneseandquackenbush2003[seneseandquackenbush2003$case2 == i, ]$t0 <-
      with(seneseandquackenbush2003, c(0, seneseandquackenbush2003[case2 == i, 
        ]$duratmo[
      1:(length(seneseandquackenbush2003[case2 == i, ]$duratmo) - 1)])),
    seneseandquackenbush2003[seneseandquackenbush2003$case2 == i, ]$t0 <- 0)
}

surv <- with(seneseandquackenbush2003[seneseandquackenbush2003$unclear == 0, ], 
  Surv(t0, duratmo, failure))
seneseandquackenbush2003.cox1 <- coxph(surv ~ negot + none + cluster(dyadid),
  data = seneseandquackenbush2003[seneseandquackenbush2003$unclear == 0, ],
  ties = 'breslow')
seneseandquackenbush2003.cox2 <- coxph(surv ~ negot + none + mindem + logrelpw +
  contigdi + decisiv1 + logrpdif + war2 + cluster(dyadid),
  data = seneseandquackenbush2003[seneseandquackenbush2003$unclear == 0, ],
  ties = 'breslow')
summary(seneseandquackenbush2003.cox1)
summary(seneseandquackenbush2003.cox2)

seneseandquackenbush2003.ph.tab1 <- zphTable(seneseandquackenbush2003.cox1, 
  digits = 3)
seneseandquackenbush2003.ph.tab2 <- zphTable(seneseandquackenbush2003.cox2,
  digits = 3)

seneseandquackenbush2003.ph.tab1
seneseandquackenbush2003.ph.tab1[, c(3, 6, 9, 12)]
seneseandquackenbush2003.ph.tab2
seneseandquackenbush2003.ph.tab2[, c(3, 6, 9, 12)]

seneseandquackenbush2003.var.labs1 <- rownames(seneseandquackenbush2003.ph.tab1)
seneseandquackenbush2003.var.labs.pretty1 <- c('Negotiated Settlement',
  'No Settlement')
seneseandquackenbush2003.var.labs2 <- rownames(seneseandquackenbush2003.ph.tab2)
seneseandquackenbush2003.var.labs.pretty2 <- c('Negotiated Settlement',
  'No Settlement', 'Minimum Democracy', 'ln(Relative Capabilities)',
  'ln(Chng. in Rel. Capabil.)', 'Geographic Contiguity',
  'Decisive Outcome', 'War')

star.tab.temp1 <- matrix(rep(NA, 4 * nrow(seneseandquackenbush2003.ph.tab1)),
  ncol = 4)

for ( i in 1:nrow(seneseandquackenbush2003.ph.tab1) ) {
  for ( j in 1:4 ) {
    ifelse ( seneseandquackenbush2003.ph.tab1[i, p.val.cols[j]] < .05,
      star.tab.temp1[i, j] <- '*', star.tab.temp1[i, j] <- ' ' )
  }
}
seneseandquackenbush2003.ph.tab.pretty1 <-
  matrix(rep(NA, 4 * nrow(seneseandquackenbush2003.ph.tab1)), ncol = 4)
rownames(seneseandquackenbush2003.ph.tab.pretty1) <-
  c(seneseandquackenbush2003.var.labs.pretty1, 'GLOBAL')
colnames(seneseandquackenbush2003.ph.tab.pretty1) <- cols
for ( i in 1:nrow(seneseandquackenbush2003.ph.tab1) - 1 ) {
  for ( j in 1:4 ) {
    seneseandquackenbush2003.ph.tab.pretty1[i, j] <-
      paste(sprintf("%.3f", seneseandquackenbush2003.ph.tab1[i, rho.cols[j]]),
      star.tab.temp1[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  seneseandquackenbush2003.ph.tab.pretty1[
    nrow(seneseandquackenbush2003.ph.tab1), j] <-
      paste(sprintf("%.3f", seneseandquackenbush2003.ph.tab1[
        nrow(seneseandquackenbush2003.ph.tab1), chi.sq.cols[j]]),
        star.tab.temp1[nrow(seneseandquackenbush2003.ph.tab1), j], sep = '')
}

sink(file = 'tables/table-si-g37.txt')
  cat('Table SI.G.37', '\n\n')
  as.data.frame(seneseandquackenbush2003.ph.tab.pretty1)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.37 supplemental information', '\n\n')
  seneseandquackenbush2003.ph.tab1
sink()

zphPlot1(cox.fit = seneseandquackenbush2003.cox2, var.lab = 'negot',
  var.lab.pretty = 'Negotiated Settlement', directory = 'figures',
  fig.name = 'figure-si-g55')

zphPlots(cox.fit = seneseandquackenbush2003.cox1,
  var.labs = seneseandquackenbush2003.var.labs1,
  var.labs.pretty = seneseandquackenbush2003.var.labs.pretty1,
  directory = 'figures/supplemental/seneseandquackenbush2003/model1')

star.tab.temp2 <- matrix(rep(NA, 4 * nrow(seneseandquackenbush2003.ph.tab2)),
  ncol = 4)

for ( i in 1:nrow(seneseandquackenbush2003.ph.tab2) ) {
  for ( j in 1:4 ) {
    ifelse ( seneseandquackenbush2003.ph.tab2[i, p.val.cols[j]] < .05,
      star.tab.temp2[i, j] <- '*', star.tab.temp2[i, j] <- ' ' )
  }
}
seneseandquackenbush2003.ph.tab.pretty2 <-
  matrix(rep(NA, 4 * nrow(seneseandquackenbush2003.ph.tab2)), ncol = 4)
rownames(seneseandquackenbush2003.ph.tab.pretty2) <-
  c(seneseandquackenbush2003.var.labs.pretty2, 'GLOBAL')
colnames(seneseandquackenbush2003.ph.tab.pretty2) <- cols
for ( i in 1:nrow(seneseandquackenbush2003.ph.tab2) - 1 ) {
  for ( j in 1:4 ) {
    seneseandquackenbush2003.ph.tab.pretty2[i, j] <-
      paste(sprintf("%.3f", seneseandquackenbush2003.ph.tab2[i, rho.cols[j]]),
      star.tab.temp2[i, j], sep = '')
  }
}
for ( j in 1:4 ) {
  seneseandquackenbush2003.ph.tab.pretty2[
    nrow(seneseandquackenbush2003.ph.tab2), j] <-
      paste(sprintf("%.3f", seneseandquackenbush2003.ph.tab2[
        nrow(seneseandquackenbush2003.ph.tab2), chi.sq.cols[j]]),
        star.tab.temp2[nrow(seneseandquackenbush2003.ph.tab2), j], sep = '')
}

sink(file = 'tables/table-si-g38.txt')
  cat('Table SI.G.38', '\n\n')
  as.data.frame(seneseandquackenbush2003.ph.tab.pretty2)
  cat('\n\n\n\n\n\n\n')
  cat('Table SI.G.38 supplemental information', '\n\n')
  seneseandquackenbush2003.ph.tab2
sink()

zphPlot1(cox.fit = seneseandquackenbush2003.cox2, var.lab = 'mindem',
  var.lab.pretty = 'Minimum Democracy', directory = 'figures',
  fig.name = 'figure-si-g56')

zphPlots(cox.fit = seneseandquackenbush2003.cox2,
  var.labs = seneseandquackenbush2003.var.labs2,
  var.labs.pretty = seneseandquackenbush2003.var.labs.pretty2,
  directory = 'figures/supplemental/seneseandquackenbush2003/model2')
################################################################################

rm(list = ls())
